@@ -1,7 +1,7 @@
+from __future__ import annotations
+
 import math
 import sys
-from typing import List
-from typing import Union
 
 import numpy as np
 import shape_functions
@@ -16,8 +16,8 @@ class BaseWFG:
         S: np.ndarray,
         A: np.ndarray,
         upper_bounds: np.ndarray,
-        shapes: List[shape_functions.BaseShapeFunction],
-        transformations: List[List[transformation_functions.BaseTransformations]],
+        shapes: list[shape_functions.BaseShapeFunction],
+        transformations: list[list[transformation_functions.BaseTransformations]],
     ) -> None:
         assert all(S > 0)
         assert all((A == 0) + (A == 1))
@@ -84,11 +84,11 @@ class WFG1:
         self.domain = np.zeros((n, 2))
         self.domain[:, 1] = upper_bounds
 
-        shapes: List[shape_functions.BaseShapeFunction]
+        shapes: list[shape_functions.BaseShapeFunction]
         shapes = [shape_functions.ConvexShapeFunction(M) for _ in range(M - 1)]
         shapes.append(shape_functions.MixedConvexOrConcaveShapeFunction(M, 1, 5))
 
-        transformations: List[List[transformation_functions.BaseTransformations]]
+        transformations: list[list[transformation_functions.BaseTransformations]]
         transformations = [[] for _ in range(4)]
 
         transformations[0] = [transformation_functions.IdenticalTransformation() for _ in range(k)]
@@ -163,11 +163,11 @@ class WFG2:
         self.domain = np.zeros((n, 2))
         self.domain[:, 1] = upper_bounds
 
-        shapes: List[shape_functions.BaseShapeFunction]
+        shapes: list[shape_functions.BaseShapeFunction]
         shapes = [shape_functions.ConvexShapeFunction(M) for _ in range(M - 1)]
         shapes.append(shape_functions.DisconnectedShapeFunction(M, 1, 1, 5))
 
-        transformations: List[List[transformation_functions.BaseTransformations]]
+        transformations: list[list[transformation_functions.BaseTransformations]]
         transformations = [[] for _ in range(3)]
 
         transformations[0] = [transformation_functions.IdenticalTransformation() for _ in range(k)]
@@ -244,10 +244,10 @@ class WFG3:
         self.domain = np.zeros((n, 2))
         self.domain[:, 1] = upper_bounds
 
-        shapes: List[shape_functions.BaseShapeFunction]
+        shapes: list[shape_functions.BaseShapeFunction]
         shapes = [shape_functions.LinearShapeFunction(M) for _ in range(M)]
 
-        transformations: List[List[transformation_functions.BaseTransformations]]
+        transformations: list[list[transformation_functions.BaseTransformations]]
         transformations = [[] for _ in range(3)]
 
         transformations[0] = [transformation_functions.IdenticalTransformation() for _ in range(k)]
@@ -322,10 +322,10 @@ class WFG4:
         self.domain = np.zeros((n, 2))
         self.domain[:, 1] = upper_bounds
 
-        shapes: List[shape_functions.BaseShapeFunction]
+        shapes: list[shape_functions.BaseShapeFunction]
         shapes = [shape_functions.ConcaveShapeFunction(M) for _ in range(M)]
 
-        transformations: List[List[transformation_functions.BaseTransformations]]
+        transformations: list[list[transformation_functions.BaseTransformations]]
         transformations = [[] for _ in range(2)]
 
         transformations[0] = [
@@ -388,10 +388,10 @@ class WFG5:
         self.domain = np.zeros((n, 2))
         self.domain[:, 1] = upper_bounds
 
-        shapes: List[shape_functions.BaseShapeFunction]
+        shapes: list[shape_functions.BaseShapeFunction]
         shapes = [shape_functions.ConcaveShapeFunction(M) for _ in range(M)]
 
-        transformations: List[List[transformation_functions.BaseTransformations]]
+        transformations: list[list[transformation_functions.BaseTransformations]]
         transformations = [[] for _ in range(2)]
 
         transformations[0] = [
@@ -455,10 +455,10 @@ class WFG6:
         self.domain = np.zeros((n, 2))
         self.domain[:, 1] = upper_bounds
 
-        shapes: List[shape_functions.BaseShapeFunction]
+        shapes: list[shape_functions.BaseShapeFunction]
         shapes = [shape_functions.ConcaveShapeFunction(M) for _ in range(M)]
 
-        transformations: List[List[transformation_functions.BaseTransformations]]
+        transformations: list[list[transformation_functions.BaseTransformations]]
         transformations = [[] for _ in range(2)]
 
         transformations[0] = [transformation_functions.IdenticalTransformation() for _ in range(k)]
@@ -521,13 +521,13 @@ class WFG7:
         self.domain = np.zeros((n, 2))
         self.domain[:, 1] = upper_bounds
 
-        shapes: List[shape_functions.BaseShapeFunction]
+        shapes: list[shape_functions.BaseShapeFunction]
         shapes = [shape_functions.ConcaveShapeFunction(M) for _ in range(M)]
 
         def _input_converter0(i: int, y: np.ndarray) -> np.ndarray:
             return y[i:n]
 
-        transformations: List[List[transformation_functions.BaseTransformations]]
+        transformations: list[list[transformation_functions.BaseTransformations]]
         transformations = [[] for _ in range(3)]
 
         transformations[0] = [
@@ -604,13 +604,13 @@ class WFG8:
         self.domain = np.zeros((n, 2))
         self.domain[:, 1] = upper_bounds
 
-        shapes: List[shape_functions.BaseShapeFunction]
+        shapes: list[shape_functions.BaseShapeFunction]
         shapes = [shape_functions.ConcaveShapeFunction(M) for _ in range(M)]
 
         def _input_converter0(i: int, y: np.ndarray) -> np.ndarray:
             return y[: i - 1]
 
-        transformations: List[List[transformation_functions.BaseTransformations]]
+        transformations: list[list[transformation_functions.BaseTransformations]]
         transformations = [[] for _ in range(3)]
 
         transformations[0] = [transformation_functions.IdenticalTransformation() for _ in range(k)]
@@ -686,13 +686,13 @@ class WFG9:
         self.domain = np.zeros((n, 2))
         self.domain[:, 1] = upper_bounds
 
-        shapes: List[shape_functions.BaseShapeFunction]
+        shapes: list[shape_functions.BaseShapeFunction]
         shapes = [shape_functions.ConcaveShapeFunction(M) for _ in range(M)]
 
         def _input_converter0(i: int, y: np.ndarray) -> np.ndarray:
             return y[i:n]
 
-        transformations: List[List[transformation_functions.BaseTransformations]]
+        transformations: list[list[transformation_functions.BaseTransformations]]
         transformations = [[] for _ in range(3)]
 
         transformations[0] = [
@@ -768,12 +768,12 @@ class WFGProblem(problem.Problem):
     def __init__(self) -> None:
         super().__init__()
 
-    def create_evaluator(self, params: List[problem.Var]) -> problem.Evaluator:
+    def create_evaluator(self, params: list[problem.Var]) -> problem.Evaluator:
         return WFGEvaluator(params)
 
 
 class WFGEvaluator(problem.Evaluator):
-    def __init__(self, params: List[problem.Var]) -> None:
+    def __init__(self, params: list[problem.Var]) -> None:
         self._n_wfg = int(sys.argv[1])
         self._n_dim = int(sys.argv[2])
         self._n_obj = int(sys.argv[3])
@@ -782,7 +782,7 @@ class WFGEvaluator(problem.Evaluator):
         self._x = np.array(params)
         self._current_step = 0
 
-        self.wfg: Union[WFG1, WFG2, WFG3, WFG4, WFG5, WFG6, WFG7, WFG8, WFG9]
+        self.wfg: WFG1 | WFG2 | WFG3 | WFG4 | WFG5 | WFG6 | WFG7 | WFG8 | WFG9
         if self._n_wfg == 1:
             self.wfg = WFG1(n_arguments=self._n_dim, n_objectives=self._n_obj, k=self._k)
         elif self._n_wfg == 2:
@@ -807,7 +807,7 @@ class WFGEvaluator(problem.Evaluator):
     def current_step(self) -> int:
         return self._current_step
 
-    def evaluate(self, next_step: int) -> List[float]:
+    def evaluate(self, next_step: int) -> list[float]:
         self._current_step = 1
         v = self.wfg(self._x)
         v = v.tolist()
