@@ -15,6 +15,7 @@ from optuna.exceptions import TrialPruned
 from optuna.study import copy_study
 from optuna.study import create_study
 from optuna.study import delete_study
+from optuna.study import get_all_study_names
 from optuna.study import get_all_study_summaries
 from optuna.study import load_study
 from optuna.study import Study
@@ -28,12 +29,14 @@ __all__ = [
     "Trial",
     "TrialPruned",
     "__version__",
+    "artifacts",
     "copy_study",
     "create_study",
     "create_trial",
     "delete_study",
     "distributions",
     "exceptions",
+    "get_all_study_names",
     "get_all_study_summaries",
     "importance",
     "integration",
@@ -51,5 +54,6 @@ __all__ = [
 ]
 
 
+artifacts = _LazyImport("optuna.artifacts")
 importance = _LazyImport("optuna.importance")
 visualization = _LazyImport("optuna.visualization")
