@@ -1,8 +1,11 @@
+from __future__ import annotations
+
+from collections.abc import Callable
+from collections.abc import Sequence
 from functools import wraps
+from inspect import Parameter
 from inspect import signature
 from typing import Any
-from typing import Callable
-from typing import Sequence
 from typing import TYPE_CHECKING
 from typing import TypeVar
 import warnings
@@ -15,6 +18,21 @@ if TYPE_CHECKING:
     _T = TypeVar("_T")
 
 
+def _get_positional_arg_names(func: "Callable[_P, _T]") -> list[str]:
+    params = signature(func).parameters
+    positional_arg_names = [
+        name
+        for name, p in params.items()
+        if p.default == Parameter.empty and p.kind == p.POSITIONAL_OR_KEYWORD
+    ]
+    return positional_arg_names
+
+
+def _infer_kwargs(previous_positional_arg_names: Sequence[str], *args: Any) -> dict[str, Any]:
+    inferred_kwargs = {arg_name: val for val, arg_name in zip(args, previous_positional_arg_names)}
+    return inferred_kwargs
+
+
 def convert_positional_args(
     *,
     previous_positional_arg_names: Sequence[str],
@@ -35,10 +53,13 @@ def convert_positional_args(
 
         @wraps(func)
         def converter_wrapper(*args: Any, **kwargs: Any) -> "_T":
-            if len(args) >= 1:
+            positional_arg_names = _get_positional_arg_names(func)
+            inferred_kwargs = _infer_kwargs(previous_positional_arg_names, *args)
+            if len(inferred_kwargs) > len(positional_arg_names):
+                expected_kwds = set(inferred_kwargs) - set(positional_arg_names)
                 warnings.warn(
-                    f"{func.__name__}(): Please give all values as keyword arguments."
-                    " See https://github.com/optuna/optuna/issues/3324 for details.",
+                    f"{func.__name__}() got {expected_kwds} as positional arguments "
+                    "but they were expected to be given as keyword arguments.",
                     FutureWarning,
                     stacklevel=warning_stacklevel,
                 )
@@ -48,15 +69,16 @@ def convert_positional_args(
                     f" arguments but {len(args)} were given."
                 )
 
-            for val, arg_name in zip(args, previous_positional_arg_names):
-                # When specifying a positional argument that is not located at the end of args as
-                # a keyword argument, raise TypeError as follows by imitating the Python standard
-                # behavior.
-                if arg_name in kwargs:
-                    raise TypeError(
-                        f"{func.__name__}() got multiple values for argument '{arg_name}'."
-                    )
-                kwargs[arg_name] = val
+            duplicated_kwds = set(kwargs).intersection(inferred_kwargs)
+            if len(duplicated_kwds):
+                # When specifying positional arguments that are not located at the end of args as
+                # keyword arguments, raise TypeError as follows by imitating the Python standard
+                # behavior
+                raise TypeError(
+                    f"{func.__name__}() got multiple values for arguments {duplicated_kwds}."
+                )
+
+            kwargs.update(inferred_kwargs)
 
             return func(**kwargs)
 
