@@ -1,4 +1,3 @@
-from collections import OrderedDict
 from typing import Callable
 from typing import Dict
 from typing import List
@@ -80,7 +79,7 @@ class MeanDecreaseImpurityImportanceEvaluator(BaseImportanceEvaluator):
             params = list(distributions.keys())
         assert params is not None
         if len(params) == 0:
-            return OrderedDict()
+            return {}
 
         trials: List[FrozenTrial] = _get_filtered_trials(study, params=params, target=target)
         trans = _SearchSpaceTransform(distributions, transform_log=False, transform_step=False)
