@@ -1,3 +1,8 @@
+from __future__ import annotations
+
+from collections.abc import Callable
+from collections.abc import Iterable
+from collections.abc import Mapping
 from logging import DEBUG
 from logging import INFO
 from logging import WARNING
@@ -5,12 +10,7 @@ from numbers import Integral
 from numbers import Number
 from time import time
 from typing import Any
-from typing import Callable
-from typing import Dict
-from typing import Iterable
 from typing import List
-from typing import Mapping
-from typing import Optional
 from typing import Union
 
 import numpy as np
@@ -24,6 +24,7 @@ from optuna._experimental import experimental_class
 from optuna._imports import try_import
 from optuna.distributions import _convert_old_distribution_to_new_distribution
 from optuna.study import StudyDirection
+from optuna.terminator import report_cross_validation_scores
 from optuna.trial import FrozenTrial
 from optuna.trial import Trial
 
@@ -48,18 +49,18 @@ with try_import() as _imports:
 if not _imports.is_successful():
     BaseEstimator = object  # NOQA
 
-ArrayLikeType = Union[List, np.ndarray, "pd.Series", spmatrix]
+ArrayLikeType = Union[List, np.ndarray, "pd.Series", "spmatrix"]
 OneDimArrayLikeType = Union[List[float], np.ndarray, "pd.Series"]
-TwoDimArrayLikeType = Union[List[List[float]], np.ndarray, "pd.DataFrame", spmatrix]
-IterableType = Union[List, "pd.DataFrame", np.ndarray, "pd.Series", spmatrix, None]
+TwoDimArrayLikeType = Union[List[List[float]], np.ndarray, "pd.DataFrame", "spmatrix"]
+IterableType = Union[List, "pd.DataFrame", np.ndarray, "pd.Series", "spmatrix", None]
 IndexableType = Union[Iterable, None]
 
 _logger = logging.get_logger(__name__)
 
 
 def _check_fit_params(
-    X: TwoDimArrayLikeType, fit_params: Dict, indices: OneDimArrayLikeType
-) -> Dict:
+    X: TwoDimArrayLikeType, fit_params: dict, indices: OneDimArrayLikeType
+) -> dict:
     fit_params_validated = {}
     for key, value in fit_params.items():
         # NOTE Original implementation:
@@ -113,8 +114,8 @@ def _num_samples(x: ArrayLikeType) -> int:
 
 
 def _safe_indexing(
-    X: Union[OneDimArrayLikeType, TwoDimArrayLikeType], indices: OneDimArrayLikeType
-) -> Union[OneDimArrayLikeType, TwoDimArrayLikeType]:
+    X: OneDimArrayLikeType | TwoDimArrayLikeType, indices: OneDimArrayLikeType
+) -> OneDimArrayLikeType | TwoDimArrayLikeType:
     if X is None:
         return X
 
@@ -183,12 +184,12 @@ class _Objective:
         estimator: "sklearn.base.BaseEstimator",
         param_distributions: Mapping[str, distributions.BaseDistribution],
         X: TwoDimArrayLikeType,
-        y: Optional[Union[OneDimArrayLikeType, TwoDimArrayLikeType]],
+        y: OneDimArrayLikeType | TwoDimArrayLikeType | None,
         cv: "BaseCrossValidator",
         enable_pruning: bool,
-        error_score: Union[Number, float, str],
-        fit_params: Dict[str, Any],
-        groups: Optional[OneDimArrayLikeType],
+        error_score: Number | float | str,
+        fit_params: dict[str, Any],
+        groups: OneDimArrayLikeType | None,
         max_iter: int,
         return_train_score: bool,
         scoring: Callable[..., Number],
@@ -215,20 +216,38 @@ class _Objective:
         if self.enable_pruning:
             scores = self._cross_validate_with_pruning(trial, estimator)
         else:
-            scores = cross_validate(
-                estimator,
-                self.X,
-                self.y,
-                cv=self.cv,
-                error_score=self.error_score,
-                fit_params=self.fit_params,
-                groups=self.groups,
-                return_train_score=self.return_train_score,
-                scoring=self.scoring,
-            )
+            try:
+                scores = cross_validate(
+                    estimator,
+                    self.X,
+                    self.y,
+                    cv=self.cv,
+                    error_score=self.error_score,
+                    fit_params=self.fit_params,
+                    groups=self.groups,
+                    return_train_score=self.return_train_score,
+                    scoring=self.scoring,
+                )
+            except ValueError:
+                n_splits = self.cv.get_n_splits(self.X, self.y, self.groups)
+                fit_time = np.array([np.nan] * n_splits)
+                score_time = np.array([np.nan] * n_splits)
+                test_score = np.array(
+                    [self.error_score if self.error_score is not None else np.nan] * n_splits
+                )
+
+                scores = {
+                    "fit_time": fit_time,
+                    "score_time": score_time,
+                    "test_score": test_score,
+                }
 
         self._store_scores(trial, scores)
 
+        test_scores = scores["test_score"]
+        scores_list = test_scores if isinstance(test_scores, list) else test_scores.tolist()
+        report_cross_validation_scores(trial, scores_list)
+
         return trial.user_attrs["mean_test_score"]
 
     def _cross_validate_with_pruning(
@@ -277,7 +296,7 @@ class _Objective:
 
         return scores
 
-    def _get_params(self, trial: Trial) -> Dict[str, Any]:
+    def _get_params(self, trial: Trial) -> dict[str, Any]:
         return {
             name: trial._suggest(name, distribution)
             for name, distribution in self.param_distributions.items()
@@ -286,10 +305,10 @@ class _Objective:
     def _partial_fit_and_score(
         self,
         estimator: "sklearn.base.BaseEstimator",
-        train: List[int],
-        test: List[int],
-        partial_fit_params: Dict[str, Any],
-    ) -> List[Number]:
+        train: list[int],
+        test: list[int],
+        partial_fit_params: dict[str, Any],
+    ) -> list[Number]:
         X_train, y_train = _safe_split(estimator, self.X, self.y, train)
         X_test, y_test = _safe_split(estimator, self.X, self.y, test, train_indices=train)
 
@@ -512,13 +531,13 @@ class OptunaSearchCV(BaseEstimator):
     def best_index_(self) -> int:
         """Trial number which corresponds to the best candidate parameter setting.
 
-        Retuned value is equivant to ``optuna_search.best_trial_.number``.
+        Returned value is equivalent to ``optuna_search.best_trial_.number``.
         """
 
         return self.best_trial_.number
 
     @property
-    def best_params_(self) -> Dict[str, Any]:
+    def best_params_(self) -> dict[str, Any]:
         """Parameters of the best trial in the :class:`~optuna.study.Study`."""
 
         self._check_is_fitted()
@@ -549,6 +568,20 @@ class OptunaSearchCV(BaseEstimator):
 
         return self.best_estimator_.classes_
 
+    @property
+    def cv_results_(self) -> dict[str, Any]:
+        """A dictionary mapping a metric name to a list of
+        Cross-Validation results of all trials."""
+        cv_results_dict_in_list = [trial_.user_attrs for trial_ in self.trials_]
+        if len(cv_results_dict_in_list) == 0:
+            cv_results_list_in_dict = {}
+        else:
+            cv_results_list_in_dict = {
+                key: [dict_[key] for dict_ in cv_results_dict_in_list]
+                for key in cv_results_dict_in_list[0]
+            }
+        return cv_results_list_in_dict
+
     @property
     def n_trials_(self) -> int:
         """Actual number of trials."""
@@ -556,7 +589,7 @@ class OptunaSearchCV(BaseEstimator):
         return len(self.trials_)
 
     @property
-    def trials_(self) -> List[FrozenTrial]:
+    def trials_(self) -> list[FrozenTrial]:
         """All trials in the :class:`~optuna.study.Study`."""
 
         self._check_is_fitted()
@@ -564,7 +597,7 @@ class OptunaSearchCV(BaseEstimator):
         return self.study_.trials
 
     @property
-    def user_attrs_(self) -> Dict[str, Any]:
+    def user_attrs_(self) -> dict[str, Any]:
         """User attributes in the :class:`~optuna.study.Study`."""
 
         self._check_is_fitted()
@@ -572,7 +605,7 @@ class OptunaSearchCV(BaseEstimator):
         return self.study_.user_attrs
 
     @property
-    def decision_function(self) -> Callable[..., Union[OneDimArrayLikeType, TwoDimArrayLikeType]]:
+    def decision_function(self) -> Callable[..., OneDimArrayLikeType | TwoDimArrayLikeType]:
         """Call ``decision_function`` on the best estimator.
 
         This is available only if the underlying estimator supports
@@ -596,7 +629,7 @@ class OptunaSearchCV(BaseEstimator):
         return self.best_estimator_.inverse_transform
 
     @property
-    def predict(self) -> Callable[..., Union[OneDimArrayLikeType, TwoDimArrayLikeType]]:
+    def predict(self) -> Callable[..., OneDimArrayLikeType | TwoDimArrayLikeType]:
         """Call ``predict`` on the best estimator.
 
         This is available only if the underlying estimator supports ``predict``
@@ -676,21 +709,21 @@ class OptunaSearchCV(BaseEstimator):
         estimator: "sklearn.base.BaseEstimator",
         param_distributions: Mapping[str, distributions.BaseDistribution],
         *,
-        cv: Optional[Union[int, "BaseCrossValidator", Iterable]] = None,
+        cv: int | "BaseCrossValidator" | Iterable | None = None,
         enable_pruning: bool = False,
-        error_score: Union[Number, float, str] = np.nan,
+        error_score: Number | float | str = np.nan,
         max_iter: int = 1000,
-        n_jobs: Optional[int] = None,
-        n_trials: int = 10,
-        random_state: Optional[Union[int, np.random.RandomState]] = None,
+        n_jobs: int | None = None,
+        n_trials: int | None = 10,
+        random_state: int | np.random.RandomState | None = None,
         refit: bool = True,
         return_train_score: bool = False,
-        scoring: Optional[Union[Callable[..., float], str]] = None,
-        study: Optional[study_module.Study] = None,
-        subsample: Union[float, int] = 1.0,
-        timeout: Optional[float] = None,
+        scoring: Callable[..., float] | str | None = None,
+        study: study_module.Study | None = None,
+        subsample: float | int = 1.0,
+        timeout: float | None = None,
         verbose: int = 0,
-        callbacks: Optional[List[Callable[[study_module.Study, FrozenTrial], None]]] = None,
+        callbacks: list[Callable[[study_module.Study, FrozenTrial], None]] | None = None,
     ) -> None:
         _imports.check()
 
@@ -751,13 +784,13 @@ class OptunaSearchCV(BaseEstimator):
         if self.study is not None and self.study.direction != StudyDirection.MAXIMIZE:
             raise ValueError("direction of study must be 'maximize'.")
 
-    def _more_tags(self) -> Dict[str, bool]:
+    def _more_tags(self) -> dict[str, bool]:
         return {"non_deterministic": True, "no_validation": True}
 
     def _refit(
         self,
         X: TwoDimArrayLikeType,
-        y: Optional[Union[OneDimArrayLikeType, TwoDimArrayLikeType]] = None,
+        y: OneDimArrayLikeType | TwoDimArrayLikeType | None = None,
         **fit_params: Any,
     ) -> "OptunaSearchCV":
         n_samples = _num_samples(X)
@@ -784,8 +817,8 @@ class OptunaSearchCV(BaseEstimator):
     def fit(
         self,
         X: TwoDimArrayLikeType,
-        y: Optional[Union[OneDimArrayLikeType, TwoDimArrayLikeType]] = None,
-        groups: Optional[OneDimArrayLikeType] = None,
+        y: OneDimArrayLikeType | TwoDimArrayLikeType | None = None,
+        groups: OneDimArrayLikeType | None = None,
         **fit_params: Any,
     ) -> "OptunaSearchCV":
         """Run fit with all sets of parameters.
@@ -885,7 +918,7 @@ class OptunaSearchCV(BaseEstimator):
             callbacks=self.callbacks,
         )
 
-        _logger.info("Finished hyperparemeter search!")
+        _logger.info("Finished hyperparameter search!")
 
         if self.refit:
             self._refit(X, y, **fit_params)
@@ -897,7 +930,7 @@ class OptunaSearchCV(BaseEstimator):
     def score(
         self,
         X: TwoDimArrayLikeType,
-        y: Optional[Union[OneDimArrayLikeType, TwoDimArrayLikeType]] = None,
+        y: OneDimArrayLikeType | TwoDimArrayLikeType | None = None,
     ) -> float:
         """Return the score on the given data.
 
