@@ -16,6 +16,7 @@ from optuna.distributions import CategoricalDistribution
 from optuna.distributions import FloatDistribution
 from optuna.distributions import IntDistribution
 from optuna.samplers import BaseSampler
+from optuna.samplers._lazy_random_state import LazyRandomState
 from optuna.study import Study
 from optuna.trial import create_trial
 from optuna.trial import FrozenTrial
@@ -32,9 +33,9 @@ class _TreeNode:
     # 3. Normal node. It has a param_name and non-empty children.
 
     param_name: Optional[str] = None
-    children: Optional[Dict[Any, "_TreeNode"]] = None
+    children: Optional[Dict[float, "_TreeNode"]] = None
 
-    def expand(self, param_name: Optional[str], search_space: Iterable[Any]) -> None:
+    def expand(self, param_name: Optional[str], search_space: Iterable[float]) -> None:
         # If the node is unexpanded, expand it.
         # Otherwise, check if the node is compatible with the given search space.
         if self.children is None:
@@ -53,7 +54,7 @@ class _TreeNode:
         self.expand(None, [])
 
     def add_path(
-        self, params_and_search_spaces: Iterable[Tuple[str, Iterable[Any], Any]]
+        self, params_and_search_spaces: Iterable[Tuple[str, Iterable[float], float]]
     ) -> Optional["_TreeNode"]:
         # Add a path (i.e. a list of suggested parameters in one trial) to the tree.
         current_node = self
@@ -73,7 +74,7 @@ class _TreeNode:
             else sum(child.count_unexpanded() for child in self.children.values())
         )
 
-    def sample_child(self, rng: np.random.RandomState) -> Any:
+    def sample_child(self, rng: np.random.RandomState) -> float:
         assert self.children is not None
         # Sample an unexpanded node in the subtree uniformly, and return the first
         # parameter value in the path to the node.
@@ -130,7 +131,7 @@ class BruteForceSampler(BaseSampler):
     """
 
     def __init__(self, seed: Optional[int] = None) -> None:
-        self._rng = np.random.RandomState(seed)
+        self._rng = LazyRandomState(seed)
 
     def infer_relative_search_space(
         self, study: Study, trial: FrozenTrial
@@ -143,9 +144,10 @@ class BruteForceSampler(BaseSampler):
         return {}
 
     @staticmethod
-    def _build_tree(trials: Iterable[FrozenTrial], params: Dict[str, Any]) -> _TreeNode:
-        # Build a _TreeNode under given params from the given trials.
-        tree = _TreeNode()
+    def _populate_tree(
+        tree: _TreeNode, trials: Iterable[FrozenTrial], params: Dict[str, Any]
+    ) -> None:
+        # Populate tree under given params from the given trials.
         incomplete_leaves: List[_TreeNode] = []
         for trial in trials:
             if not all(p in trial.params and trial.params[p] == v for p, v in params.items()):
@@ -172,7 +174,6 @@ class BruteForceSampler(BaseSampler):
         for leaf in incomplete_leaves:
             if leaf.children is None:
                 leaf.set_leaf()
-        return tree
 
     def sample_independent(
         self,
@@ -185,16 +186,22 @@ class BruteForceSampler(BaseSampler):
             deepcopy=False,
             states=(
                 TrialState.COMPLETE,
+                TrialState.PRUNED,
                 TrialState.RUNNING,
+                TrialState.FAIL,
             ),
         )
-        tree = self._build_tree((t for t in trials if t.number != trial.number), trial.params)
+        tree = _TreeNode()
         candidates = _enumerate_candidates(param_distribution)
         tree.expand(param_name, candidates)
+        # Populating must happen after the initialization above to prevent `tree` from
+        # being initialized as an empty graph, which is created with n_jobs > 1
+        # where we get trials[i].params = {} for some i.
+        self._populate_tree(tree, (t for t in trials if t.number != trial.number), trial.params)
         if tree.count_unexpanded() == 0:
-            return param_distribution.to_external_repr(self._rng.choice(candidates))
+            return param_distribution.to_external_repr(self._rng.rng.choice(candidates))
         else:
-            return param_distribution.to_external_repr(tree.sample_child(self._rng))
+            return param_distribution.to_external_repr(tree.sample_child(self._rng.rng))
 
     def after_trial(
         self,
@@ -207,10 +214,14 @@ class BruteForceSampler(BaseSampler):
             deepcopy=False,
             states=(
                 TrialState.COMPLETE,
+                TrialState.PRUNED,
                 TrialState.RUNNING,
+                TrialState.FAIL,
             ),
         )
-        tree = self._build_tree(
+        tree = _TreeNode()
+        self._populate_tree(
+            tree,
             (
                 t
                 if t.number != trial.number
@@ -229,7 +240,7 @@ class BruteForceSampler(BaseSampler):
             study.stop()
 
 
-def _enumerate_candidates(param_distribution: BaseDistribution) -> Sequence[Any]:
+def _enumerate_candidates(param_distribution: BaseDistribution) -> Sequence[float]:
     if isinstance(param_distribution, FloatDistribution):
         if param_distribution.step is None:
             raise ValueError(
