@@ -1,19 +1,30 @@
 from typing import Union
 
-from optuna._callbacks import RetryFailedTrialCallback  # NOQA
+from optuna._callbacks import RetryFailedTrialCallback
 from optuna.storages._base import BaseStorage
 from optuna.storages._cached_storage import _CachedStorage
 from optuna.storages._heartbeat import fail_stale_trials
 from optuna.storages._in_memory import InMemoryStorage
+from optuna.storages._journal.base import BaseJournalLogStorage
+from optuna.storages._journal.file import JournalFileOpenLock
+from optuna.storages._journal.file import JournalFileStorage
+from optuna.storages._journal.file import JournalFileSymlinkLock
+from optuna.storages._journal.redis import JournalRedisStorage
+from optuna.storages._journal.storage import JournalStorage
 from optuna.storages._rdb.storage import RDBStorage
-from optuna.storages._redis import RedisStorage
 
 
 __all__ = [
     "BaseStorage",
+    "BaseJournalLogStorage",
     "InMemoryStorage",
     "RDBStorage",
-    "RedisStorage",
+    "JournalStorage",
+    "JournalFileSymlinkLock",
+    "JournalFileOpenLock",
+    "JournalFileStorage",
+    "JournalRedisStorage",
+    "RetryFailedTrialCallback",
     "_CachedStorage",
     "fail_stale_trials",
 ]
@@ -26,9 +37,10 @@ def get_storage(storage: Union[None, str, BaseStorage]) -> BaseStorage:
         return InMemoryStorage()
     if isinstance(storage, str):
         if storage.startswith("redis"):
-            return RedisStorage(storage)
-        else:
-            return _CachedStorage(RDBStorage(storage))
+            raise ValueError(
+                "RedisStorage is removed at Optuna v3.1.0. Please use JournalRedisStorage instead."
+            )
+        return _CachedStorage(RDBStorage(storage))
     elif isinstance(storage, RDBStorage):
         return _CachedStorage(storage)
     else:
