@@ -8,7 +8,6 @@ from typing import Tuple
 from sqlalchemy import asc
 from sqlalchemy import case
 from sqlalchemy import CheckConstraint
-from sqlalchemy import Column
 from sqlalchemy import DateTime
 from sqlalchemy import desc
 from sqlalchemy import Enum
@@ -32,6 +31,14 @@ except ImportError:
     # TODO(c-bata): Remove this after dropping support for SQLAlchemy v1.3 or prior.
     from sqlalchemy.ext.declarative import declarative_base
 
+try:
+    from sqlalchemy.orm import mapped_column
+
+    _Column = mapped_column
+except ImportError:
+    # TODO(Shinichi): Remove this after dropping support for SQLAlchemy<2.0.
+    from sqlalchemy import Column as _Column  # type: ignore[assignment]
+
 
 # Don't modify this version number anymore.
 # The schema management functionality has been moved to alembic.
@@ -49,8 +56,10 @@ BaseModel: Any = declarative_base()
 
 class StudyModel(BaseModel):
     __tablename__ = "studies"
-    study_id = Column(Integer, primary_key=True)
-    study_name = Column(String(MAX_INDEXED_STRING_LENGTH), index=True, unique=True, nullable=False)
+    study_id = _Column(Integer, primary_key=True)
+    study_name = _Column(
+        String(MAX_INDEXED_STRING_LENGTH), index=True, unique=True, nullable=False
+    )
 
     @classmethod
     def find_or_raise_by_id(
@@ -85,10 +94,10 @@ class StudyModel(BaseModel):
 class StudyDirectionModel(BaseModel):
     __tablename__ = "study_directions"
     __table_args__: Any = (UniqueConstraint("study_id", "objective"),)
-    study_direction_id = Column(Integer, primary_key=True)
-    direction = Column(Enum(StudyDirection), nullable=False)
-    study_id = Column(Integer, ForeignKey("studies.study_id"), nullable=False)
-    objective = Column(Integer, nullable=False)
+    study_direction_id = _Column(Integer, primary_key=True)
+    direction = _Column(Enum(StudyDirection), nullable=False)
+    study_id = _Column(Integer, ForeignKey("studies.study_id"), nullable=False)
+    objective = _Column(Integer, nullable=False)
 
     study = orm.relationship(
         StudyModel, backref=orm.backref("directions", cascade="all, delete-orphan")
@@ -102,10 +111,10 @@ class StudyDirectionModel(BaseModel):
 class StudyUserAttributeModel(BaseModel):
     __tablename__ = "study_user_attributes"
     __table_args__: Any = (UniqueConstraint("study_id", "key"),)
-    study_user_attribute_id = Column(Integer, primary_key=True)
-    study_id = Column(Integer, ForeignKey("studies.study_id"))
-    key = Column(String(MAX_INDEXED_STRING_LENGTH))
-    value_json = Column(Text())
+    study_user_attribute_id = _Column(Integer, primary_key=True)
+    study_id = _Column(Integer, ForeignKey("studies.study_id"))
+    key = _Column(String(MAX_INDEXED_STRING_LENGTH))
+    value_json = _Column(Text())
 
     study = orm.relationship(
         StudyModel, backref=orm.backref("user_attributes", cascade="all, delete-orphan")
@@ -134,10 +143,10 @@ class StudyUserAttributeModel(BaseModel):
 class StudySystemAttributeModel(BaseModel):
     __tablename__ = "study_system_attributes"
     __table_args__: Any = (UniqueConstraint("study_id", "key"),)
-    study_system_attribute_id = Column(Integer, primary_key=True)
-    study_id = Column(Integer, ForeignKey("studies.study_id"))
-    key = Column(String(MAX_INDEXED_STRING_LENGTH))
-    value_json = Column(Text())
+    study_system_attribute_id = _Column(Integer, primary_key=True)
+    study_id = _Column(Integer, ForeignKey("studies.study_id"))
+    key = _Column(String(MAX_INDEXED_STRING_LENGTH))
+    value_json = _Column(Text())
 
     study = orm.relationship(
         StudyModel, backref=orm.backref("system_attributes", cascade="all, delete-orphan")
@@ -165,15 +174,15 @@ class StudySystemAttributeModel(BaseModel):
 
 class TrialModel(BaseModel):
     __tablename__ = "trials"
-    trial_id = Column(Integer, primary_key=True)
+    trial_id = _Column(Integer, primary_key=True)
     # No `UniqueConstraint` is put on the `number` columns although it in practice is constrained
     # to be unique. This is to reduce code complexity as table-level locking would be required
     # otherwise. See https://github.com/optuna/optuna/pull/939#discussion_r387447632.
-    number = Column(Integer)
-    study_id = Column(Integer, ForeignKey("studies.study_id"), index=True)
-    state = Column(Enum(TrialState), nullable=False)
-    datetime_start = Column(DateTime)
-    datetime_complete = Column(DateTime)
+    number = _Column(Integer)
+    study_id = _Column(Integer, ForeignKey("studies.study_id"), index=True)
+    state = _Column(Enum(TrialState), nullable=False)
+    datetime_start = _Column(DateTime)
+    datetime_complete = _Column(DateTime)
 
     study = orm.relationship(
         StudyModel, backref=orm.backref("trials", cascade="all, delete-orphan")
@@ -273,10 +282,10 @@ class TrialModel(BaseModel):
 class TrialUserAttributeModel(BaseModel):
     __tablename__ = "trial_user_attributes"
     __table_args__: Any = (UniqueConstraint("trial_id", "key"),)
-    trial_user_attribute_id = Column(Integer, primary_key=True)
-    trial_id = Column(Integer, ForeignKey("trials.trial_id"))
-    key = Column(String(MAX_INDEXED_STRING_LENGTH))
-    value_json = Column(Text())
+    trial_user_attribute_id = _Column(Integer, primary_key=True)
+    trial_id = _Column(Integer, ForeignKey("trials.trial_id"))
+    key = _Column(String(MAX_INDEXED_STRING_LENGTH))
+    value_json = _Column(Text())
 
     trial = orm.relationship(
         TrialModel, backref=orm.backref("user_attributes", cascade="all, delete-orphan")
@@ -305,10 +314,10 @@ class TrialUserAttributeModel(BaseModel):
 class TrialSystemAttributeModel(BaseModel):
     __tablename__ = "trial_system_attributes"
     __table_args__: Any = (UniqueConstraint("trial_id", "key"),)
-    trial_system_attribute_id = Column(Integer, primary_key=True)
-    trial_id = Column(Integer, ForeignKey("trials.trial_id"))
-    key = Column(String(MAX_INDEXED_STRING_LENGTH))
-    value_json = Column(Text())
+    trial_system_attribute_id = _Column(Integer, primary_key=True)
+    trial_id = _Column(Integer, ForeignKey("trials.trial_id"))
+    key = _Column(String(MAX_INDEXED_STRING_LENGTH))
+    value_json = _Column(Text())
 
     trial = orm.relationship(
         TrialModel, backref=orm.backref("system_attributes", cascade="all, delete-orphan")
@@ -337,11 +346,11 @@ class TrialSystemAttributeModel(BaseModel):
 class TrialParamModel(BaseModel):
     __tablename__ = "trial_params"
     __table_args__: Any = (UniqueConstraint("trial_id", "param_name"),)
-    param_id = Column(Integer, primary_key=True)
-    trial_id = Column(Integer, ForeignKey("trials.trial_id"))
-    param_name = Column(String(MAX_INDEXED_STRING_LENGTH))
-    param_value = Column(Float(precision=FLOAT_PRECISION))
-    distribution_json = Column(Text())
+    param_id = _Column(Integer, primary_key=True)
+    trial_id = _Column(Integer, ForeignKey("trials.trial_id"))
+    param_name = _Column(String(MAX_INDEXED_STRING_LENGTH))
+    param_value = _Column(Float(precision=FLOAT_PRECISION))
+    distribution_json = _Column(Text())
 
     trial = orm.relationship(
         TrialModel, backref=orm.backref("params", cascade="all, delete-orphan")
@@ -408,11 +417,11 @@ class TrialValueModel(BaseModel):
 
     __tablename__ = "trial_values"
     __table_args__: Any = (UniqueConstraint("trial_id", "objective"),)
-    trial_value_id = Column(Integer, primary_key=True)
-    trial_id = Column(Integer, ForeignKey("trials.trial_id"), nullable=False)
-    objective = Column(Integer, nullable=False)
-    value = Column(Float(precision=FLOAT_PRECISION), nullable=True)
-    value_type = Column(Enum(TrialValueType), nullable=False)
+    trial_value_id = _Column(Integer, primary_key=True)
+    trial_id = _Column(Integer, ForeignKey("trials.trial_id"), nullable=False)
+    objective = _Column(Integer, nullable=False)
+    value = _Column(Float(precision=FLOAT_PRECISION), nullable=True)
+    value_type = _Column(Enum(TrialValueType), nullable=False)
 
     trial = orm.relationship(
         TrialModel, backref=orm.backref("values", cascade="all, delete-orphan")
@@ -474,11 +483,11 @@ class TrialIntermediateValueModel(BaseModel):
 
     __tablename__ = "trial_intermediate_values"
     __table_args__: Any = (UniqueConstraint("trial_id", "step"),)
-    trial_intermediate_value_id = Column(Integer, primary_key=True)
-    trial_id = Column(Integer, ForeignKey("trials.trial_id"), nullable=False)
-    step = Column(Integer, nullable=False)
-    intermediate_value = Column(Float(precision=FLOAT_PRECISION), nullable=True)
-    intermediate_value_type = Column(Enum(TrialIntermediateValueType), nullable=False)
+    trial_intermediate_value_id = _Column(Integer, primary_key=True)
+    trial_id = _Column(Integer, ForeignKey("trials.trial_id"), nullable=False)
+    step = _Column(Integer, nullable=False)
+    intermediate_value = _Column(Float(precision=FLOAT_PRECISION), nullable=True)
+    intermediate_value_type = _Column(Enum(TrialIntermediateValueType), nullable=False)
 
     trial = orm.relationship(
         TrialModel, backref=orm.backref("intermediate_values", cascade="all, delete-orphan")
@@ -541,9 +550,9 @@ class TrialIntermediateValueModel(BaseModel):
 class TrialHeartbeatModel(BaseModel):
     __tablename__ = "trial_heartbeats"
     __table_args__: Any = (UniqueConstraint("trial_id"),)
-    trial_heartbeat_id = Column(Integer, primary_key=True)
-    trial_id = Column(Integer, ForeignKey("trials.trial_id"), nullable=False)
-    heartbeat = Column(DateTime, nullable=False, default=func.current_timestamp())
+    trial_heartbeat_id = _Column(Integer, primary_key=True)
+    trial_id = _Column(Integer, ForeignKey("trials.trial_id"), nullable=False)
+    heartbeat = _Column(DateTime, nullable=False, default=func.current_timestamp())
 
     trial = orm.relationship(
         TrialModel, backref=orm.backref("heartbeats", cascade="all, delete-orphan")
@@ -560,12 +569,11 @@ class VersionInfoModel(BaseModel):
     __tablename__ = "version_info"
     # setting check constraint to ensure the number of rows is at most 1
     __table_args__: Any = (CheckConstraint("version_info_id=1"),)
-    version_info_id = Column(Integer, primary_key=True, autoincrement=False, default=1)
-    schema_version = Column(Integer)
-    library_version = Column(String(MAX_VERSION_LENGTH))
+    version_info_id = _Column(Integer, primary_key=True, autoincrement=False, default=1)
+    schema_version = _Column(Integer)
+    library_version = _Column(String(MAX_VERSION_LENGTH))
 
     @classmethod
-    def find(cls, session: orm.Session) -> "VersionInfoModel":
+    def find(cls, session: orm.Session) -> Optional["VersionInfoModel"]:
         version_info = session.query(cls).one_or_none()
-
         return version_info
