@@ -4,6 +4,7 @@ from optuna.terminator.erroreval import CrossValidationErrorEvaluator
 from optuna.terminator.erroreval import report_cross_validation_scores
 from optuna.terminator.erroreval import StaticErrorEvaluator
 from optuna.terminator.improvement.evaluator import BaseImprovementEvaluator
+from optuna.terminator.improvement.evaluator import BestValueStagnationEvaluator
 from optuna.terminator.improvement.evaluator import RegretBoundEvaluator
 from optuna.terminator.terminator import BaseTerminator
 from optuna.terminator.terminator import Terminator
@@ -16,6 +17,7 @@ __all__ = [
     "report_cross_validation_scores",
     "StaticErrorEvaluator",
     "BaseImprovementEvaluator",
+    "BestValueStagnationEvaluator",
     "RegretBoundEvaluator",
     "BaseTerminator",
     "Terminator",
