@@ -5,7 +5,10 @@ from typing import Callable
 from typing import cast
 from typing import NamedTuple
 
+from optuna.distributions import CategoricalChoiceType
+from optuna.distributions import CategoricalDistribution
 from optuna.logging import get_logger
+from optuna.samplers._base import _CONSTRAINTS_KEY
 from optuna.study import Study
 from optuna.trial import FrozenTrial
 from optuna.trial import TrialState
@@ -13,7 +16,6 @@ from optuna.visualization._plotly_imports import _imports
 from optuna.visualization._utils import _check_plot_args
 from optuna.visualization._utils import _filter_nonfinite
 from optuna.visualization._utils import _is_log_scale
-from optuna.visualization._utils import _is_numerical
 
 
 if _imports.is_successful():
@@ -32,6 +34,8 @@ class _SliceSubplotInfo(NamedTuple):
     trial_numbers: list[int]
     is_log: bool
     is_numerical: bool
+    constraints: list[bool]
+    x_labels: tuple[CategoricalChoiceType, ...] | None
 
 
 class _SlicePlotInfo(NamedTuple):
@@ -39,12 +43,19 @@ class _SlicePlotInfo(NamedTuple):
     subplots: list[_SliceSubplotInfo]
 
 
+class _PlotValues(NamedTuple):
+    x: list[Any]
+    y: list[float]
+    trial_numbers: list[int]
+
+
 def _get_slice_subplot_info(
     trials: list[FrozenTrial],
     param: str,
     target: Callable[[FrozenTrial], float] | None,
     log_scale: bool,
     numerical: bool,
+    x_labels: tuple[CategoricalChoiceType, ...] | None,
 ) -> _SliceSubplotInfo:
     if target is None:
 
@@ -53,15 +64,28 @@ def _get_slice_subplot_info(
 
         target = _target
 
-    return _SliceSubplotInfo(
+    plot_info = _SliceSubplotInfo(
         param_name=param,
-        x=[t.params[param] for t in trials if param in t.params],
-        y=[target(t) for t in trials if param in t.params],
-        trial_numbers=[t.number for t in trials if param in t.params],
+        x=[],
+        y=[],
+        trial_numbers=[],
         is_log=log_scale,
         is_numerical=numerical,
+        x_labels=x_labels,
+        constraints=[],
     )
 
+    for t in trials:
+        if param not in t.params:
+            continue
+        plot_info.x.append(t.params[param])
+        plot_info.y.append(target(t))
+        plot_info.trial_numbers.append(t.number)
+        constraints = t.system_attrs.get(_CONSTRAINTS_KEY)
+        plot_info.constraints.append(constraints is None or all([x <= 0.0 for x in constraints]))
+
+    return plot_info
+
 
 def _get_slice_plot_info(
     study: Study,
@@ -80,6 +104,18 @@ def _get_slice_plot_info(
         return _SlicePlotInfo(target_name, [])
 
     all_params = {p_name for t in trials for p_name in t.params.keys()}
+
+    distributions = {}
+    for trial in trials:
+        for param_name, distribution in trial.distributions.items():
+            if param_name not in distributions:
+                distributions[param_name] = distribution
+
+    x_labels = {}
+    for param_name, distribution in distributions.items():
+        if isinstance(distribution, CategoricalDistribution):
+            x_labels[param_name] = distribution.choices
+
     if params is None:
         sorted_params = sorted(all_params)
     else:
@@ -96,7 +132,8 @@ def _get_slice_plot_info(
                 param=param,
                 target=target,
                 log_scale=_is_log_scale(trials, param),
-                numerical=_is_numerical(trials, param),
+                numerical=not isinstance(distributions[param], CategoricalDistribution),
+                x_labels=x_labels.get(param),
             )
             for param in sorted_params
         ],
@@ -164,10 +201,14 @@ def _get_slice_plot(info: _SlicePlotInfo) -> "go.Figure":
     if len(info.subplots) == 0:
         return go.Figure(data=[], layout=layout)
     elif len(info.subplots) == 1:
-        figure = go.Figure(data=[_generate_slice_subplot(info.subplots[0])], layout=layout)
+        figure = go.Figure(data=_generate_slice_subplot(info.subplots[0]), layout=layout)
         figure.update_xaxes(title_text=info.subplots[0].param_name)
         figure.update_yaxes(title_text=info.target_name)
-        if info.subplots[0].is_log:
+        if not info.subplots[0].is_numerical:
+            figure.update_xaxes(
+                type="category", categoryorder="array", categoryarray=info.subplots[0].x_labels
+            )
+        elif info.subplots[0].is_log:
             figure.update_xaxes(type="log")
     else:
         figure = make_subplots(rows=1, cols=len(info.subplots), shared_yaxes=True)
@@ -175,14 +216,23 @@ def _get_slice_plot(info: _SlicePlotInfo) -> "go.Figure":
         showscale = True  # showscale option only needs to be specified once.
         for column_index, subplot_info in enumerate(info.subplots, start=1):
             trace = _generate_slice_subplot(subplot_info)
-            trace.update(marker={"showscale": showscale})  # showscale's default is True.
+            trace[0].update(marker={"showscale": showscale})  # showscale's default is True.
             if showscale:
                 showscale = False
-            figure.add_trace(trace, row=1, col=column_index)
+            for t in trace:
+                figure.add_trace(t, row=1, col=column_index)
             figure.update_xaxes(title_text=subplot_info.param_name, row=1, col=column_index)
             if column_index == 1:
                 figure.update_yaxes(title_text=info.target_name, row=1, col=column_index)
-            if subplot_info.is_log:
+            if not subplot_info.is_numerical:
+                figure.update_xaxes(
+                    type="category",
+                    categoryorder="array",
+                    categoryarray=subplot_info.x_labels,
+                    row=1,
+                    col=column_index,
+                )
+            elif subplot_info.is_log:
                 figure.update_xaxes(type="log", row=1, col=column_index)
         if len(info.subplots) > 3:
             # Ensure that each subplot has a minimum width without relying on autusizing.
@@ -191,23 +241,54 @@ def _get_slice_plot(info: _SlicePlotInfo) -> "go.Figure":
     return figure
 
 
-def _generate_slice_subplot(subplot_info: _SliceSubplotInfo) -> "Scatter":
-    x = [x if x is not None else "None" for x in subplot_info.x]
-    y = [y if y is not None else "None" for y in subplot_info.y]
+def _generate_slice_subplot(subplot_info: _SliceSubplotInfo) -> list[Scatter]:
+    trace = []
 
-    return go.Scatter(
-        x=x,
-        y=y,
-        mode="markers",
-        marker={
-            "line": {"width": 0.5, "color": "Grey"},
-            "color": subplot_info.trial_numbers,
-            "colorscale": COLOR_SCALE,
-            "colorbar": {
-                "title": "Trial",
-                "x": 1.0,  # Offset the colorbar position with a fixed width `xpad`.
-                "xpad": 40,
+    feasible = _PlotValues([], [], [])
+    infeasible = _PlotValues([], [], [])
+
+    for x, y, num, c in zip(
+        subplot_info.x, subplot_info.y, subplot_info.trial_numbers, subplot_info.constraints
+    ):
+        if x is not None or x != "None" or y is not None or y != "None":
+            if c:
+                feasible.x.append(x)
+                feasible.y.append(y)
+                feasible.trial_numbers.append(num)
+            else:
+                infeasible.x.append(x)
+                infeasible.y.append(y)
+    trace.append(
+        go.Scatter(
+            x=feasible.x,
+            y=feasible.y,
+            mode="markers",
+            name="Feasible Trial",
+            marker={
+                "line": {"width": 0.5, "color": "Grey"},
+                "color": feasible.trial_numbers,
+                "colorscale": COLOR_SCALE,
+                "colorbar": {
+                    "title": "Trial",
+                    "x": 1.0,  # Offset the colorbar position with a fixed width `xpad`.
+                    "xpad": 40,
+                },
             },
-        },
-        showlegend=False,
+            showlegend=False,
+        )
     )
+    if len(infeasible.x) > 0:
+        trace.append(
+            go.Scatter(
+                x=infeasible.x,
+                y=infeasible.y,
+                mode="markers",
+                name="Infeasible Trial",
+                marker={
+                    "color": "#cccccc",
+                },
+                showlegend=False,
+            )
+        )
+
+    return trace
