@@ -1,7 +1,7 @@
 from __future__ import annotations
 
-from typing import Callable
-from typing import Sequence
+from collections.abc import Callable
+from collections.abc import Sequence
 
 from optuna._experimental import experimental_func
 from optuna.logging import get_logger
