@@ -9,8 +9,6 @@ def test_import() -> None:
     from optuna.integration import DaskStorage  # NOQA
     from optuna.integration import lightgbm  # NOQA
     from optuna.integration import LightGBMPruningCallback  # NOQA
-    from optuna.integration import mxnet  # NOQA
-    from optuna.integration import MXNetPruningCallback  # NOQA
     from optuna.integration import xgboost  # NOQA
     from optuna.integration import XGBoostPruningCallback  # NOQA
 
@@ -23,10 +21,8 @@ def test_module_attributes() -> None:
 
     assert hasattr(optuna.integration, "dask")
     assert hasattr(optuna.integration, "lightgbm")
-    assert hasattr(optuna.integration, "mxnet")
     assert hasattr(optuna.integration, "xgboost")
     assert hasattr(optuna.integration, "LightGBMPruningCallback")
-    assert hasattr(optuna.integration, "MXNetPruningCallback")
     assert hasattr(optuna.integration, "XGBoostPruningCallback")
 
     with pytest.raises(AttributeError):
