@@ -193,17 +193,17 @@ def test_study_system_attr_for_population_cache() -> None:
 
 def test_reseed_rng() -> None:
     sampler = multi_objective.samplers.NSGAIIMultiObjectiveSampler(population_size=10)
-    original_random_state = sampler._rng.get_state()
-    original_random_sampler_random_state = sampler._random_sampler._sampler._rng.get_state()
+    original_random_state = sampler._rng.rng.get_state()
+    original_random_sampler_random_state = sampler._random_sampler._sampler._rng.rng.get_state()
 
     with patch.object(
         sampler._random_sampler, "reseed_rng", wraps=sampler._random_sampler.reseed_rng
     ) as mock_object:
         sampler.reseed_rng()
         assert mock_object.call_count == 1
-    assert str(original_random_state) != str(sampler._rng.get_state())
+    assert str(original_random_state) != str(sampler._rng.rng.get_state())
     assert str(original_random_sampler_random_state) != str(
-        sampler._random_sampler._sampler._rng.get_state()
+        sampler._random_sampler._sampler._rng.rng.get_state()
     )
 
 
