@@ -1,13 +1,12 @@
+from __future__ import annotations
+
 from collections import Counter
+from collections.abc import Callable
+from collections.abc import Sequence
 import copy
 import itertools
 from typing import Any
-from typing import Callable
-from typing import Dict
-from typing import List
-from typing import Optional
-from typing import Sequence
-from typing import Tuple
+from unittest.mock import MagicMock
 from unittest.mock import Mock
 from unittest.mock import patch
 import warnings
@@ -24,6 +23,7 @@ from optuna.distributions import IntDistribution
 from optuna.samplers import BaseSampler
 from optuna.samplers import NSGAIISampler
 from optuna.samplers._base import _CONSTRAINTS_KEY
+from optuna.samplers._lazy_random_state import LazyRandomState
 from optuna.samplers.nsgaii import BaseCrossover
 from optuna.samplers.nsgaii import BLXAlphaCrossover
 from optuna.samplers.nsgaii import SBXCrossover
@@ -31,9 +31,18 @@ from optuna.samplers.nsgaii import SPXCrossover
 from optuna.samplers.nsgaii import UNDXCrossover
 from optuna.samplers.nsgaii import UniformCrossover
 from optuna.samplers.nsgaii import VSBXCrossover
+from optuna.samplers.nsgaii._after_trial_strategy import NSGAIIAfterTrialStrategy
+from optuna.samplers.nsgaii._child_generation_strategy import NSGAIIChildGenerationStrategy
 from optuna.samplers.nsgaii._crossover import _inlined_categorical_uniform_crossover
-from optuna.samplers.nsgaii._sampler import _constrained_dominates
-from optuna.samplers.nsgaii._sampler import _fast_non_dominated_sort
+from optuna.samplers.nsgaii._dominates import _constrained_dominates
+from optuna.samplers.nsgaii._dominates import _validate_constraints
+from optuna.samplers.nsgaii._elite_population_selection_strategy import (
+    NSGAIIElitePopulationSelectionStrategy,
+)
+from optuna.samplers.nsgaii._elite_population_selection_strategy import _calc_crowding_distance
+from optuna.samplers.nsgaii._elite_population_selection_strategy import _crowding_distance_sort
+from optuna.samplers.nsgaii._elite_population_selection_strategy import _fast_non_dominated_sort
+from optuna.samplers.nsgaii._sampler import _GENERATION_KEY
 from optuna.study._multi_objective import _dominates
 from optuna.study._study_direction import StudyDirection
 from optuna.trial import FrozenTrial
@@ -74,9 +83,10 @@ def test_population_size() -> None:
         # Less than 2.
         NSGAIISampler(population_size=1)
 
-    with pytest.raises(TypeError):
-        # Not an integer.
-        NSGAIISampler(population_size=2.5)  # type: ignore
+    with pytest.raises(ValueError):
+        mock_crossover = MagicMock(spec=BaseCrossover)
+        mock_crossover.configure_mock(n_parents=3)
+        NSGAIISampler(population_size=2, crossover=mock_crossover)
 
 
 def test_mutation_prob() -> None:
@@ -115,9 +125,15 @@ def test_swapping_prob() -> None:
     with pytest.raises(ValueError):
         NSGAIISampler(swapping_prob=1.1)
 
+    with pytest.raises(ValueError):
+        UniformCrossover(swapping_prob=-0.5)
+
+    with pytest.raises(ValueError):
+        UniformCrossover(swapping_prob=1.1)
+
 
 @pytest.mark.parametrize("choices", [[-1, 0, 1], [True, False]])
-def test_crossover_casting(choices: List[Any]) -> None:
+def test_crossover_casting(choices: list[Any]) -> None:
     str_choices = list(map(str, choices))
 
     def objective(trial: optuna.Trial) -> Sequence[float]:
@@ -215,7 +231,7 @@ def test_constraints_func_nan() -> None:
     ],
 )
 def test_constrained_dominates_feasible_vs_feasible(
-    direction1: StudyDirection, direction2: StudyDirection, constraints_list: List[List[float]]
+    direction1: StudyDirection, direction2: StudyDirection, constraints_list: list[list[float]]
 ) -> None:
     directions = [direction1, direction2]
     # Check all pairs of trials consisting of these values, i.e.,
@@ -297,7 +313,7 @@ def test_constrained_dominates_infeasible_vs_infeasible(direction: StudyDirectio
     #
 
     # Check all pairs of these constraints.
-    constraints_infeasible_sorted: List[List[List[float]]]
+    constraints_infeasible_sorted: list[list[list[float]]]
     constraints_infeasible_sorted = [
         # These constraints have violation 1.
         [[1, -inf], [1, -1], [1, 0], [0, 1], [-1, 1], [-inf, 1]],
@@ -347,9 +363,9 @@ def test_constrained_dominates_infeasible_vs_infeasible(direction: StudyDirectio
 
 
 def _assert_population_per_rank(
-    trials: List[FrozenTrial],
-    direction: List[StudyDirection],
-    population_per_rank: List[List[FrozenTrial]],
+    trials: list[FrozenTrial],
+    direction: list[StudyDirection],
+    population_per_rank: list[list[FrozenTrial]],
 ) -> None:
     # Check that the number of trials do not change.
     flattened = [trial for rank in population_per_rank for trial in rank]
@@ -382,7 +398,7 @@ def test_fast_non_dominated_sort_no_constraints(
     values = [[v1, v2] for v1 in value_list for v2 in value_list]
 
     trials = [_create_frozen_trial(i, v) for i, v in enumerate(values)]
-    population_per_rank = _fast_non_dominated_sort(copy.copy(trials), directions)
+    population_per_rank = _fast_non_dominated_sort(copy.copy(trials), directions, _dominates)
     _assert_population_per_rank(trials, directions, population_per_rank)
 
 
@@ -399,17 +415,15 @@ def test_fast_non_dominated_sort_with_constraints() -> None:
     ]
     directions = [StudyDirection.MINIMIZE, StudyDirection.MAXIMIZE]
     population_per_rank = _fast_non_dominated_sort(
-        copy.copy(trials), directions, constraints_func=lambda _: [0]
+        copy.copy(trials), directions, _constrained_dominates
     )
     _assert_population_per_rank(trials, directions, population_per_rank)
 
 
-def test_fast_non_dominated_sort_with_nan_constraint() -> None:
-    directions = [StudyDirection.MINIMIZE, StudyDirection.MINIMIZE]
+def test_validate_constraints() -> None:
     with pytest.raises(ValueError):
-        _fast_non_dominated_sort(
+        _validate_constraints(
             [_create_frozen_trial(0, [1], [0, float("nan")])],
-            directions,
             constraints_func=lambda _: [0],
         )
 
@@ -428,7 +442,7 @@ def test_fast_non_dominated_sort_with_nan_constraint() -> None:
     ],
 )
 def test_fast_non_dominated_sort_missing_constraint_values(
-    values_and_constraints: List[Tuple[List[float], List[float]]]
+    values_and_constraints: list[tuple[list[float], list[float]]]
 ) -> None:
     with warnings.catch_warnings():
         warnings.simplefilter("ignore", optuna.exceptions.ExperimentalWarning)
@@ -441,7 +455,7 @@ def test_fast_non_dominated_sort_missing_constraint_values(
 
         with pytest.warns(UserWarning):
             population_per_rank = _fast_non_dominated_sort(
-                copy.copy(trials), list(directions), constraints_func=lambda _: [0]
+                copy.copy(trials), list(directions), _constrained_dominates
             )
         _assert_population_per_rank(trials, list(directions), population_per_rank)
 
@@ -451,8 +465,8 @@ def test_fast_non_dominated_sort_empty(n_dims: int) -> None:
     for directions in itertools.product(
         [StudyDirection.MINIMIZE, StudyDirection.MAXIMIZE], repeat=n_dims
     ):
-        trials: List[FrozenTrial] = []
-        population_per_rank = _fast_non_dominated_sort(trials, list(directions))
+        trials: list[FrozenTrial] = []
+        population_per_rank = _fast_non_dominated_sort(trials, list(directions), _dominates)
         assert population_per_rank == []
 
 
@@ -484,9 +498,9 @@ def test_fast_non_dominated_sort_empty(n_dims: int) -> None:
         ),
     ],
 )
-def test_calc_crowding_distance(values: List[List[float]], expected_dist: List[float]) -> None:
+def test_calc_crowding_distance(values: list[list[float]], expected_dist: list[float]) -> None:
     trials = [_create_frozen_trial(i, value) for i, value in enumerate(values)]
-    crowding_dist = optuna.samplers.nsgaii._sampler._calc_crowding_distance(trials)
+    crowding_dist = _calc_crowding_distance(trials)
     for i in range(len(trials)):
         assert _nan_equal(crowding_dist[i], expected_dist[i]), i
 
@@ -501,11 +515,11 @@ def test_calc_crowding_distance(values: List[List[float]], expected_dist: List[f
         [[float("-inf")], [1], [2]],
     ],
 )
-def test_crowding_distance_sort(values: List[List[float]]) -> None:
+def test_crowding_distance_sort(values: list[list[float]]) -> None:
     """Checks that trials are sorted by the values of `_calc_crowding_distance`."""
     trials = [_create_frozen_trial(i, value) for i, value in enumerate(values)]
-    crowding_dist = optuna.samplers.nsgaii._sampler._calc_crowding_distance(trials)
-    optuna.samplers.nsgaii._sampler._crowding_distance_sort(trials)
+    crowding_dist = _calc_crowding_distance(trials)
+    _crowding_distance_sort(trials)
     sorted_dist = [crowding_dist[t.number] for t in trials]
     assert sorted_dist == sorted(sorted_dist, reverse=True)
 
@@ -516,7 +530,7 @@ def test_study_system_attr_for_population_cache() -> None:
 
     def get_cached_entries(
         study: optuna.study.Study,
-    ) -> List[Tuple[int, List[int]]]:
+    ) -> list[tuple[int, list[int]]]:
         study_system_attrs = study._storage.get_study_system_attrs(study._study_id)
         return [
             v
@@ -546,9 +560,24 @@ def test_constraints_func_experimental_warning() -> None:
         NSGAIISampler(constraints_func=lambda _: [0])
 
 
+def test_elite_population_selection_strategy_experimental_warning() -> None:
+    with pytest.warns(optuna.exceptions.ExperimentalWarning):
+        NSGAIISampler(elite_population_selection_strategy=lambda study, population: [])
+
+
+def test_child_generation_strategy_experimental_warning() -> None:
+    with pytest.warns(optuna.exceptions.ExperimentalWarning):
+        NSGAIISampler(child_generation_strategy=lambda study, search_space, parent_population: {})
+
+
+def test_after_trial_strategy_experimental_warning() -> None:
+    with pytest.warns(optuna.exceptions.ExperimentalWarning):
+        NSGAIISampler(after_trial_strategy=lambda study, trial, state, value: None)
+
+
 # TODO(ohta): Consider to move this utility function to `optuna.testing` module.
 def _create_frozen_trial(
-    number: int, values: Sequence[float], constraints: Optional[Sequence[float]] = None
+    number: int, values: Sequence[float], constraints: Sequence[float] | None = None
 ) -> optuna.trial.FrozenTrial:
     trial = optuna.trial.create_trial(
         state=optuna.trial.TrialState.COMPLETE,
@@ -560,6 +589,171 @@ def _create_frozen_trial(
     return trial
 
 
+def test_elite_population_selection_strategy_invalid_value() -> None:
+    with pytest.raises(ValueError):
+        NSGAIIElitePopulationSelectionStrategy(population_size=1)
+
+
+@pytest.mark.parametrize(
+    "objectives, expected_elite_population",
+    [
+        (
+            [[1.0, 4.0], [2.0, 3.0], [3.0, 2.0], [4.0, 1.0]],
+            [[1.0, 4.0], [2.0, 3.0], [3.0, 2.0], [4.0, 1.0]],
+        ),
+        (
+            [[1.0, 2.0], [2.0, 1.0], [3.0, 3.0], [4.0, 4.0]],
+            [[1.0, 2.0], [2.0, 1.0], [3.0, 3.0], [4.0, 4.0]],
+        ),
+        (
+            [[1.0, 2.0], [2.0, 1.0], [5.0, 3.0], [3.0, 5.0], [4.0, 4.0]],
+            [[1.0, 2.0], [2.0, 1.0], [5.0, 3.0], [3.0, 5.0]],
+        ),
+    ],
+)
+def test_elite_population_selection_strategy_result(
+    objectives: list[list[float]],
+    expected_elite_population: list[list[float]],
+) -> None:
+    population_size = 4
+    elite_population_selection_strategy = NSGAIIElitePopulationSelectionStrategy(
+        population_size=population_size
+    )
+    study = optuna.create_study(directions=["minimize", "minimize"])
+    study.add_trials([optuna.create_trial(values=values) for values in objectives])
+    elite_population_values = [
+        trial.values for trial in elite_population_selection_strategy(study, study.get_trials())
+    ]
+    assert len(elite_population_values) == population_size
+    for values in elite_population_values:
+        assert values in expected_elite_population
+
+
+@pytest.mark.parametrize(
+    "mutation_prob,crossover,crossover_prob,swapping_prob",
+    [
+        (1.2, UniformCrossover(), 0.9, 0.5),
+        (-0.2, UniformCrossover(), 0.9, 0.5),
+        (None, UniformCrossover(), 1.2, 0.5),
+        (None, UniformCrossover(), -0.2, 0.5),
+        (None, UniformCrossover(), 0.9, 1.2),
+        (None, UniformCrossover(), 0.9, -0.2),
+        (None, 3, 0.9, 0.5),
+    ],
+)
+def test_child_generation_strategy_invalid_value(
+    mutation_prob: float,
+    crossover: BaseCrossover | int,
+    crossover_prob: float,
+    swapping_prob: float,
+) -> None:
+    with pytest.raises(ValueError):
+        NSGAIIChildGenerationStrategy(
+            mutation_prob=mutation_prob,
+            crossover=crossover,  # type: ignore[arg-type]
+            crossover_prob=crossover_prob,
+            swapping_prob=swapping_prob,
+            rng=LazyRandomState(),
+        )
+
+
+@pytest.mark.parametrize(
+    "mutation_prob,child_params",
+    [(0.0, {"x": 1.0, "y": 0.0}), (1.0, {})],
+)
+def test_child_generation_strategy_mutation_prob(
+    mutation_prob: int, child_params: dict[str, float]
+) -> None:
+    child_generation_strategy = NSGAIIChildGenerationStrategy(
+        crossover_prob=0.0,
+        crossover=UniformCrossover(),
+        mutation_prob=mutation_prob,
+        swapping_prob=0.5,
+        rng=LazyRandomState(seed=1),
+    )
+    study = MagicMock(spec=optuna.study.Study)
+    search_space = MagicMock(spec=dict)
+    search_space.keys.return_value = ["x", "y"]
+    parent_population = [
+        optuna.trial.create_trial(
+            params={"x": 1.0, "y": 0},
+            distributions={
+                "x": FloatDistribution(0, 10),
+                "y": CategoricalDistribution([-1, 0, 1]),
+            },
+            value=5.0,
+        )
+    ]
+    assert child_generation_strategy(study, search_space, parent_population) == child_params
+
+
+def test_child_generation_strategy_generation_key() -> None:
+    n_params = 2
+
+    def objective(trial: optuna.Trial) -> list[float]:
+        xs = [trial.suggest_float(f"x{dim}", -10, 10) for dim in range(n_params)]
+        return xs
+
+    mock_func = MagicMock(spec=Callable, return_value={"x0": 0.0, "x1": 1.1})
+    with warnings.catch_warnings():
+        warnings.simplefilter("ignore", optuna.exceptions.ExperimentalWarning)
+        study = optuna.create_study(
+            sampler=NSGAIISampler(population_size=2, child_generation_strategy=mock_func),
+            directions=["minimize", "minimize"],
+        )
+    study.optimize(objective, n_trials=3)
+    assert mock_func.call_count == 1
+    for i, trial in enumerate(study.get_trials()):
+        if i < 2:
+            assert trial.system_attrs[_GENERATION_KEY] == 0
+        elif i == 2:
+            assert trial.system_attrs[_GENERATION_KEY] == 1
+
+
+@patch(
+    "optuna.samplers.nsgaii._child_generation_strategy.perform_crossover",
+    return_value={"x": 3.0, "y": 2.0},
+)
+def test_child_generation_strategy_crossover_prob(mock_func: MagicMock) -> None:
+    study = MagicMock(spec=optuna.study.Study)
+    search_space = MagicMock(spec=dict)
+    search_space.keys.return_value = ["x", "y"]
+    parent_population = [
+        optuna.trial.create_trial(
+            params={"x": 1.0, "y": 0},
+            distributions={
+                "x": FloatDistribution(0, 10),
+                "y": CategoricalDistribution([-1, 0, 1]),
+            },
+            value=5.0,
+        )
+    ]
+    child_generation_strategy_always_not_crossover = NSGAIIChildGenerationStrategy(
+        crossover_prob=0.0,
+        crossover=UniformCrossover(),
+        mutation_prob=None,
+        swapping_prob=0.5,
+        rng=LazyRandomState(seed=1),
+    )
+    assert child_generation_strategy_always_not_crossover(
+        study, search_space, parent_population
+    ) == {"x": 1.0}
+    assert mock_func.call_count == 0
+
+    child_generation_strategy_always_crossover = NSGAIIChildGenerationStrategy(
+        crossover_prob=1.0,
+        crossover=UniformCrossover(),
+        mutation_prob=0.0,
+        swapping_prob=0.5,
+        rng=LazyRandomState(),
+    )
+    assert child_generation_strategy_always_crossover(study, search_space, parent_population) == {
+        "x": 3.0,
+        "y": 2.0,
+    }
+    assert mock_func.call_count == 1
+
+
 def test_call_after_trial_of_random_sampler() -> None:
     sampler = NSGAIISampler()
     study = optuna.create_study(sampler=sampler)
@@ -570,6 +764,34 @@ def test_call_after_trial_of_random_sampler() -> None:
         assert mock_object.call_count == 1
 
 
+def test_call_after_trial_of_after_trial_strategy() -> None:
+    sampler = NSGAIISampler()
+    study = optuna.create_study(sampler=sampler)
+    with patch.object(sampler, "_after_trial_strategy") as mock_object:
+        study.optimize(lambda _: 1.0, n_trials=1)
+        assert mock_object.call_count == 1
+
+
+@patch("optuna.samplers.nsgaii._after_trial_strategy._process_constraints_after_trial")
+def test_nsgaii_after_trial_strategy(mock_func: MagicMock) -> None:
+    def constraints_func(_: FrozenTrial) -> Sequence[float]:
+        return (float("nan"),)
+
+    state = optuna.trial.TrialState.FAIL
+    study = optuna.create_study()
+    trial = optuna.trial.create_trial(state=state)
+
+    after_trial_strategy_without_constrains = NSGAIIAfterTrialStrategy()
+    after_trial_strategy_without_constrains(study, trial, state)
+    assert mock_func.call_count == 0
+
+    after_trial_strategy_with_constrains = NSGAIIAfterTrialStrategy(
+        constraints_func=constraints_func
+    )
+    after_trial_strategy_with_constrains(study, trial, state)
+    assert mock_func.call_count == 1
+
+
 parametrize_nsga2_sampler = pytest.mark.parametrize(
     "sampler_class",
     [
@@ -657,7 +879,7 @@ def test_crossover_numerical_distribution(crossover: BaseCrossover) -> None:
 
 
 def test_crossover_inlined_categorical_distribution() -> None:
-    search_space: Dict[str, BaseDistribution] = {
+    search_space: dict[str, BaseDistribution] = {
         "x": CategoricalDistribution(choices=["a", "c"]),
         "y": CategoricalDistribution(choices=["b", "d"]),
     }
@@ -731,7 +953,7 @@ def test_crossover_deterministic(
     crossover: BaseCrossover, rand_value: float, expected_params: np.ndarray
 ) -> None:
     study = optuna.study.create_study()
-    search_space: Dict[str, BaseDistribution] = {
+    search_space: dict[str, BaseDistribution] = {
         "x": FloatDistribution(1, 10),
         "y": FloatDistribution(1, 10),
     }
@@ -749,7 +971,7 @@ def test_crossover_deterministic(
     def _normal(*args: Any, **kwargs: Any) -> Any:
         if kwargs.get("size") is None:
             return rand_value
-        return np.full(kwargs.get("size"), rand_value)  # type: ignore
+        return np.full(kwargs.get("size"), rand_value)  # type: ignore[arg-type]
 
     rng = Mock()
     rng.rand = Mock(side_effect=_rand)
