@@ -49,7 +49,7 @@ def objective_test_upgrade(trial: optuna.trial.Trial) -> float:
     x = trial.suggest_float("x", -5, 5)  # optuna==0.9.0 does not have suggest_float.
     y = trial.suggest_int("y", 0, 10)
     z = trial.suggest_categorical("z", [-5, 0, 5])
-    trial.set_system_attr("a", 0)
+    trial.storage.set_trial_system_attr(trial._trial_id, "a", 0)
     trial.set_user_attr("b", 1)
     trial.report(0.5, step=0)
     return x**2 + y**2 + z**2
@@ -59,7 +59,7 @@ def mo_objective_test_upgrade(trial: optuna.trial.Trial) -> Tuple[float, float]:
     x = trial.suggest_float("x", -5, 5)
     y = trial.suggest_int("y", 0, 10)
     z = trial.suggest_categorical("z", [-5, 0, 5])
-    trial.set_system_attr("a", 0)
+    trial.storage.set_trial_system_attr(trial._trial_id, "a", 0)
     trial.set_user_attr("b", 1)
     return x, x**2 + y**2 + z**2
 
