@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import copy
 from datetime import datetime
 import pickle
@@ -773,6 +775,25 @@ def test_get_all_trials(storage_mode: str) -> None:
             storage.get_all_trials(non_existent_study_id)
 
 
+@pytest.mark.parametrize("storage_mode", STORAGE_MODES)
+@pytest.mark.parametrize("param_names", [["a", "b"], ["b", "a"]])
+def test_get_all_trials_params_order(storage_mode: str, param_names: list[str]) -> None:
+    # We don't actually require that all storages to preserve the order of parameters,
+    # but all current implementations do, so we test this property.
+    with StorageSupplier(storage_mode) as storage:
+        study_id = storage.create_new_study(directions=[StudyDirection.MINIMIZE])
+        trial_id = storage.create_new_trial(
+            study_id, optuna.trial.create_trial(state=TrialState.RUNNING)
+        )
+        for param_name in param_names:
+            storage.set_trial_param(
+                trial_id, param_name, 1.0, distribution=FloatDistribution(0.0, 2.0)
+            )
+
+        trials = storage.get_all_trials(study_id)
+        assert list(trials[0].params.keys()) == param_names
+
+
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_get_all_trials_deepcopy_option(storage_mode: str) -> None:
     with StorageSupplier(storage_mode) as storage:
@@ -829,6 +850,35 @@ def test_get_all_trials_state_option(storage_mode: str) -> None:
             assert len(trials) == 0
 
 
+@pytest.mark.parametrize("storage_mode", STORAGE_MODES)
+def test_get_all_trials_not_modified(storage_mode: str) -> None:
+    with StorageSupplier(storage_mode) as storage:
+        _, study_to_trials = _setup_studies(storage, n_study=2, n_trial=20, seed=48)
+
+        for study_id in study_to_trials.keys():
+            trials = storage.get_all_trials(study_id, deepcopy=False)
+            deepcopied_trials = copy.deepcopy(trials)
+
+            for trial in trials:
+                if not trial.state.is_finished():
+                    storage.set_trial_param(trial._trial_id, "paramX", 0, FloatDistribution(0, 1))
+                    storage.set_trial_user_attr(trial._trial_id, "usr_attrX", 0)
+                    storage.set_trial_system_attr(trial._trial_id, "sys_attrX", 0)
+
+                if trial.state == TrialState.RUNNING:
+                    if trial.number % 3 == 0:
+                        storage.set_trial_state_values(trial._trial_id, TrialState.COMPLETE, [0])
+                    elif trial.number % 3 == 1:
+                        storage.set_trial_intermediate_value(trial._trial_id, 0, 0)
+                        storage.set_trial_state_values(trial._trial_id, TrialState.PRUNED, [0])
+                    else:
+                        storage.set_trial_state_values(trial._trial_id, TrialState.FAIL)
+                elif trial.state == TrialState.WAITING:
+                    storage.set_trial_state_values(trial._trial_id, TrialState.RUNNING)
+
+            assert trials == deepcopied_trials
+
+
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
 def test_get_n_trials(storage_mode: str) -> None:
     with StorageSupplier(storage_mode) as storage:
