@@ -1,15 +1,18 @@
-from collections import OrderedDict
 import json
 import os
+import platform
 import re
 import subprocess
 from subprocess import CalledProcessError
+import tempfile
 from typing import Any
 from typing import Callable
 from typing import Optional
 from typing import Tuple
+from unittest.mock import MagicMock
 from unittest.mock import patch
 
+import fakeredis
 import numpy as np
 from pandas import Timedelta
 from pandas import Timestamp
@@ -20,6 +23,9 @@ import optuna
 import optuna.cli
 from optuna.exceptions import CLIUsageError
 from optuna.exceptions import ExperimentalWarning
+from optuna.storages import JournalFileStorage
+from optuna.storages import JournalRedisStorage
+from optuna.storages import JournalStorage
 from optuna.storages import RDBStorage
 from optuna.storages._base import DEFAULT_STUDY_NAME_PREFIX
 from optuna.study import StudyDirection
@@ -65,8 +71,11 @@ def _parse_output(output: str, output_format: str) -> Any:
         For table format, a list of dict formatted rows.
         For JSON or YAML format, a list or a dict corresponding to ``output``.
     """
-
-    if output_format == "table":
+    if output_format == "value":
+        # Currently, _parse_output with output_format="value" is used only for
+        # `study-names` command.
+        return [{"name": values} for values in output.split(os.linesep)]
+    elif output_format == "table":
         rows = output.split(os.linesep)
         assert all(len(rows[0]) == len(row) for row in rows)
         # Check ruled lines.
@@ -75,7 +84,7 @@ def _parse_output(output: str, output_format: str) -> Any:
         keys = [r.strip() for r in rows[1].split("|")[1:-1]]
         ret = []
         for record in rows[3:-1]:
-            attrs = OrderedDict()
+            attrs = {}
             for key, attr in zip(keys, record.split("|")[1:-1]):
                 attrs[key] = attr.strip()
             ret.append(attrs)
@@ -291,6 +300,71 @@ def test_study_set_user_attr_command() -> None:
         assert all(study_user_attrs[k] == v for k, v in example_attrs.items())
 
 
+@pytest.mark.skip_coverage
+@pytest.mark.parametrize("output_format", (None, "table", "json", "yaml"))
+def test_study_names_command(output_format: Optional[str]) -> None:
+    with StorageSupplier("sqlite") as storage:
+        assert isinstance(storage, RDBStorage)
+        storage_url = str(storage.engine.url)
+
+        expected_study_names = ["study-names-test1", "study-names-test2"]
+        expected_column_name = "name"
+
+        # Create a study.
+        command = [
+            "optuna",
+            "create-study",
+            "--storage",
+            storage_url,
+            "--study-name",
+            expected_study_names[0],
+        ]
+        subprocess.check_output(command)
+
+        # Get study names.
+        command = ["optuna", "study-names", "--storage", storage_url]
+        if output_format is not None:
+            command += ["--format", output_format]
+        output = str(subprocess.check_output(command).decode().strip())
+        study_names = _parse_output(output, output_format or "value")
+
+        # Check user_attrs are not printed.
+        assert len(study_names) == 1
+        assert study_names[0]["name"] == expected_study_names[0]
+
+        # Create another study.
+        command = [
+            "optuna",
+            "create-study",
+            "--storage",
+            storage_url,
+            "--study-name",
+            expected_study_names[1],
+        ]
+        subprocess.check_output(command)
+
+        # Get study names.
+        command = ["optuna", "study-names", "--storage", storage_url]
+        if output_format is not None:
+            command += ["--format", output_format]
+        output = str(subprocess.check_output(command).decode().strip())
+        study_names = _parse_output(output, output_format or "value")
+
+        assert len(study_names) == 2
+        for i, study_name in enumerate(study_names):
+            assert list(study_name.keys()) == [expected_column_name]
+            assert study_name["name"] == expected_study_names[i]
+
+
+@pytest.mark.skip_coverage
+def test_study_names_command_without_storage_url() -> None:
+    with pytest.raises(subprocess.CalledProcessError):
+        subprocess.check_output(
+            ["optuna", "study-names", "--study-name", "dummy_study"],
+            env={k: v for k, v in os.environ.items() if k != "OPTUNA_STORAGE"},
+        )
+
+
 @pytest.mark.skip_coverage
 @pytest.mark.parametrize("output_format", (None, "table", "json", "yaml"))
 def test_studies_command(output_format: Optional[str]) -> None:
@@ -1049,6 +1123,51 @@ def test_check_storage_url() -> None:
         optuna.cli._check_storage_url(None)
 
 
+@pytest.mark.skipif(platform.system() == "Windows", reason="Skip on Windows")
+@patch("optuna.storages._journal.redis.redis")
+def test_get_storage_without_storage_class(mock_redis: MagicMock) -> None:
+    with tempfile.NamedTemporaryFile(suffix=".db") as fp:
+        storage = optuna.cli._get_storage(f"sqlite:///{fp.name}", storage_class=None)
+        assert isinstance(storage, RDBStorage)
+
+    with tempfile.NamedTemporaryFile(suffix=".log") as fp:
+        storage = optuna.cli._get_storage(fp.name, storage_class=None)
+        assert isinstance(storage, JournalStorage)
+        assert isinstance(storage._backend, JournalFileStorage)
+
+    mock_redis.Redis = fakeredis.FakeRedis
+    storage = optuna.cli._get_storage("redis://localhost:6379", storage_class=None)
+    assert isinstance(storage, JournalStorage)
+    assert isinstance(storage._backend, JournalRedisStorage)
+
+    with pytest.raises(CLIUsageError):
+        optuna.cli._get_storage("./file-not-found.log", storage_class=None)
+
+
+@pytest.mark.skipif(platform.system() == "Windows", reason="Skip on Windows")
+@patch("optuna.storages._journal.redis.redis")
+def test_get_storage_with_storage_class(mock_redis: MagicMock) -> None:
+    with tempfile.NamedTemporaryFile(suffix=".db") as fp:
+        storage = optuna.cli._get_storage(f"sqlite:///{fp.name}", storage_class=None)
+        assert isinstance(storage, RDBStorage)
+
+    with tempfile.NamedTemporaryFile(suffix=".log") as fp:
+        storage = optuna.cli._get_storage(fp.name, storage_class="JournalFileStorage")
+        assert isinstance(storage, JournalStorage)
+        assert isinstance(storage._backend, JournalFileStorage)
+
+    mock_redis.Redis = fakeredis.FakeRedis
+    storage = optuna.cli._get_storage(
+        "redis:///localhost:6379", storage_class="JournalRedisStorage"
+    )
+    assert isinstance(storage, JournalStorage)
+    assert isinstance(storage._backend, JournalRedisStorage)
+
+    with pytest.raises(CLIUsageError):
+        with tempfile.NamedTemporaryFile(suffix=".db") as fp:
+            optuna.cli._get_storage(f"sqlite:///{fp.name}", storage_class="InMemoryStorage")
+
+
 @pytest.mark.skip_coverage
 def test_storage_upgrade_command() -> None:
     with StorageSupplier("sqlite") as storage:
@@ -1066,6 +1185,16 @@ def test_storage_upgrade_command() -> None:
         subprocess.check_call(command)
 
 
+@pytest.mark.skip_coverage
+def test_storage_upgrade_command_with_invalid_url() -> None:
+    with StorageSupplier("sqlite") as storage:
+        assert isinstance(storage, RDBStorage)
+
+        command = ["optuna", "storage", "upgrade", "--storage", "invalid-storage-url"]
+        with pytest.raises(CalledProcessError):
+            subprocess.check_call(command)
+
+
 @pytest.mark.skip_coverage
 @pytest.mark.parametrize(
     "direction,directions,sampler,sampler_kwargs,output_format",
