@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import datetime
 import math
 from typing import Any
@@ -128,11 +130,11 @@ def test_check_distribution_suggest_discrete_uniform(storage_mode: str) -> None:
         assert len([r for r in record if r.category != FutureWarning]) == 1
 
         with pytest.raises(ValueError):
-            trial.suggest_int("x", 10, 20, 2)
+            trial.suggest_int("x", 10, 20, step=2)
 
         trial = Trial(study, study._storage.create_new_trial(study._study_id))
         with pytest.raises(ValueError):
-            trial.suggest_int("x", 10, 20, 2)
+            trial.suggest_int("x", 10, 20, step=2)
 
 
 @pytest.mark.parametrize("storage_mode", STORAGE_MODES)
@@ -613,13 +615,6 @@ def test_report_warning() -> None:
         trial.report(1, 1)
 
 
-def test_study_id() -> None:
-    study = create_study()
-    trial = Trial(study, study._storage.create_new_trial(study._study_id))
-
-    assert trial._study_id == trial.study._study_id
-
-
 def test_suggest_with_multi_objectives() -> None:
     study = create_study(directions=["maximize", "maximize"])
 
@@ -693,8 +688,8 @@ def test_lazy_trial_system_attrs(storage_mode: str) -> None:
         # Then, we create the instance for each method, and test the first and second use.
 
         system_attrs = _LazyTrialSystemAttrs(trial._trial_id, storage)
-        assert system_attrs == {"int": 0, "str": "A"}  # type: ignore[comparison-overlap]
-        assert system_attrs == {"int": 0, "str": "A"}  # type: ignore[comparison-overlap]
+        assert system_attrs == {"int": 0, "str": "A"}
+        assert system_attrs == {"int": 0, "str": "A"}
 
         system_attrs = _LazyTrialSystemAttrs(trial._trial_id, storage)
         assert len(system_attrs) == 2
@@ -711,3 +706,14 @@ def test_lazy_trial_system_attrs(storage_mode: str) -> None:
         system_attrs = _LazyTrialSystemAttrs(trial._trial_id, storage)
         assert set(system_attrs.items()) == {("int", 0), ("str", "A")}
         assert set(system_attrs.items()) == {("int", 0), ("str", "A")}
+
+
+@pytest.mark.parametrize("positional_args_names", [[], ["step"], ["step", "log"]])
+def test_suggest_int_positional_args(positional_args_names: list[str]) -> None:
+    # If log is specified as positional, step must also be provided as positional.
+    study = optuna.create_study()
+    trial = study.ask()
+    kwargs = dict(step=1, log=False)
+    args = [kwargs[name] for name in positional_args_names]
+    # No error should not be raised even if the coding style is old.
+    trial.suggest_int("x", -1, 1, *args)
