@@ -1,5 +1,6 @@
+from __future__ import annotations
+
 from io import BytesIO
-from typing import List
 
 import pytest
 
@@ -12,7 +13,7 @@ from tests.visualization_tests.test_optimization_history import optimization_his
 @pytest.mark.parametrize("target_name", ["Objective Value", "Target Name"])
 @pytest.mark.parametrize("info_list", optimization_history_info_lists)
 def test_get_optimization_history_plot(
-    target_name: str, info_list: List[_OptimizationHistoryInfo]
+    target_name: str, info_list: list[_OptimizationHistoryInfo]
 ) -> None:
     figure = _get_optimization_history_plot(info_list, target_name=target_name)
     assert figure.get_ylabel() == target_name
@@ -24,3 +25,4 @@ def test_get_optimization_history_plot(
     legends = [legend.get_text() for legend in figure.legend().get_texts()]
     assert sorted(legends) == sorted(expected_legends)
     plt.savefig(BytesIO())
+    plt.close()
