@@ -1,14 +1,11 @@
-import datetime
+from __future__ import annotations
+
 from io import BytesIO
-from textwrap import dedent
 from typing import Any
 from typing import Callable
-from typing import List
-from typing import Optional
 from typing import Sequence
 import warnings
 
-import numpy as np
 import pytest
 
 import optuna
@@ -17,11 +14,9 @@ from optuna import create_trial
 from optuna.distributions import FloatDistribution
 from optuna.study.study import Study
 from optuna.trial import FrozenTrial
-from optuna.trial import TrialState
 from optuna.visualization import plot_pareto_front
 import optuna.visualization._pareto_front
 from optuna.visualization._pareto_front import _get_pareto_front_info
-from optuna.visualization._pareto_front import _make_hovertext
 from optuna.visualization._pareto_front import _ParetoFrontInfo
 from optuna.visualization._plotly_imports import go
 from optuna.visualization._utils import COLOR_SCALE
@@ -51,7 +46,6 @@ def test_get_pareto_front_info_infer_n_targets() -> None:
 
 
 def create_study_2d() -> Study:
-
     study = optuna.create_study(directions=["minimize", "minimize"])
 
     study.enqueue_trial({"x": 1, "y": 2})
@@ -63,7 +57,6 @@ def create_study_2d() -> Study:
 
 
 def create_study_3d() -> Study:
-
     study = optuna.create_study(directions=["minimize", "minimize", "minimize"])
 
     study.enqueue_trial({"x": 1, "y": 2})
@@ -81,16 +74,20 @@ def create_study_3d() -> Study:
 @pytest.mark.parametrize("axis_order", [None, [0, 1], [1, 0]])
 @pytest.mark.parametrize("targets", [None, lambda t: (t.values[0], t.values[1])])
 @pytest.mark.parametrize("target_names", [None, ["Foo", "Bar"]])
+@pytest.mark.parametrize("metric_names", [None, ["v0", "v1"]])
 def test_get_pareto_front_info_unconstrained(
     include_dominated_trials: bool,
-    axis_order: Optional[List[int]],
-    targets: Optional[Callable[[FrozenTrial], Sequence[float]]],
-    target_names: Optional[List[str]],
+    axis_order: list[int] | None,
+    targets: Callable[[FrozenTrial], Sequence[float]] | None,
+    target_names: list[str] | None,
+    metric_names: list[str] | None,
 ) -> None:
     if axis_order is not None and targets is not None:
         pytest.skip("skip using both axis_order and targets")
 
     study = create_study_2d()
+    if metric_names is not None:
+        study.set_metric_names(metric_names)
     trials = study.get_trials(deepcopy=False)
     with warnings.catch_warnings():
         warnings.filterwarnings("ignore", category=FutureWarning)
@@ -104,7 +101,7 @@ def test_get_pareto_front_info_unconstrained(
 
     assert info == _ParetoFrontInfo(
         n_targets=2,
-        target_names=target_names or ["Objective 0", "Objective 1"],
+        target_names=target_names or metric_names or ["Objective 0", "Objective 1"],
         best_trials_with_values=[(trials[2], [0, 2]), (trials[3], [1, 0])],
         non_best_trials_with_values=[(trials[0], [1, 2]), (trials[1], [1, 1])]
         if include_dominated_trials
@@ -120,16 +117,20 @@ def test_get_pareto_front_info_unconstrained(
 @pytest.mark.parametrize("axis_order", [None, [0, 1], [1, 0]])
 @pytest.mark.parametrize("targets", [None, lambda t: (t.values[0], t.values[1])])
 @pytest.mark.parametrize("target_names", [None, ["Foo", "Bar"]])
+@pytest.mark.parametrize("metric_names", [None, ["v0", "v1"]])
 def test_get_pareto_front_info_constrained(
     include_dominated_trials: bool,
-    axis_order: Optional[List[int]],
-    targets: Optional[Callable[[FrozenTrial], Sequence[float]]],
-    target_names: Optional[List[str]],
+    axis_order: list[int] | None,
+    targets: Callable[[FrozenTrial], Sequence[float]] | None,
+    target_names: list[str] | None,
+    metric_names: list[str] | None,
 ) -> None:
     if axis_order is not None and targets is not None:
         pytest.skip("skip using both axis_order and targets")
 
     study = create_study_2d()
+    if metric_names is not None:
+        study.set_metric_names(metric_names)
     trials = study.get_trials(deepcopy=False)
 
     # (x, y) = (1, 0) is infeasible; others are feasible.
@@ -149,7 +150,7 @@ def test_get_pareto_front_info_constrained(
 
     assert info == _ParetoFrontInfo(
         n_targets=2,
-        target_names=target_names or ["Objective 0", "Objective 1"],
+        target_names=target_names or metric_names or ["Objective 0", "Objective 1"],
         best_trials_with_values=[(trials[1], [1, 1]), (trials[2], [0, 2])],
         non_best_trials_with_values=[(trials[0], [1, 2])] if include_dominated_trials else [],
         infeasible_trials_with_values=[(trials[3], [1, 0])],
@@ -165,9 +166,9 @@ def test_get_pareto_front_info_constrained(
 @pytest.mark.parametrize("target_names", [None, ["Foo", "Bar", "Baz"]])
 def test_get_pareto_front_info_3d(
     include_dominated_trials: bool,
-    axis_order: Optional[List[int]],
-    targets: Optional[Callable[[FrozenTrial], Sequence[float]]],
-    target_names: Optional[List[str]],
+    axis_order: list[int] | None,
+    targets: Callable[[FrozenTrial], Sequence[float]] | None,
+    target_names: list[str] | None,
 ) -> None:
     if axis_order is not None and targets is not None:
         pytest.skip("skip using both axis_order and targets")
@@ -211,7 +212,7 @@ def test_get_pareto_front_info_invalid_number_of_target_names() -> None:
 def test_get_pareto_front_info_unsupported_dimensions(
     n_dims: int,
     include_dominated_trials: bool,
-    constraints_func: Optional[Callable[[FrozenTrial], Sequence[float]]],
+    constraints_func: Callable[[FrozenTrial], Sequence[float]] | None,
 ) -> None:
     study = optuna.create_study(directions=["minimize"] * n_dims)
     with pytest.raises(ValueError):
@@ -227,9 +228,9 @@ def test_get_pareto_front_info_unsupported_dimensions(
 @pytest.mark.parametrize("include_dominated_trials", [False, True])
 @pytest.mark.parametrize("constraints_func", [None, lambda _: [-1.0]])
 def test_get_pareto_front_info_invalid_axis_order(
-    axis_order: List[int],
+    axis_order: list[int],
     include_dominated_trials: bool,
-    constraints_func: Optional[Callable[[FrozenTrial], Sequence[float]]],
+    constraints_func: Callable[[FrozenTrial], Sequence[float]] | None,
 ) -> None:
     study = optuna.create_study(directions=["minimize", "minimize"])
     with pytest.raises(ValueError):
@@ -245,7 +246,7 @@ def test_get_pareto_front_info_invalid_axis_order(
 @pytest.mark.parametrize("constraints_func", [None, lambda _: [-1.0]])
 def test_get_pareto_front_info_invalid_target_values(
     include_dominated_trials: bool,
-    constraints_func: Optional[Callable[[FrozenTrial], Sequence[float]]],
+    constraints_func: Callable[[FrozenTrial], Sequence[float]] | None,
 ) -> None:
     study = optuna.create_study(directions=["minimize", "minimize"])
     study.optimize(lambda _: [0, 0], n_trials=3)
@@ -263,7 +264,7 @@ def test_get_pareto_front_info_invalid_target_values(
 @pytest.mark.parametrize("constraints_func", [None, lambda _: [-1.0]])
 def test_get_pareto_front_info_using_axis_order_and_targets(
     include_dominated_trials: bool,
-    constraints_func: Optional[Callable[[FrozenTrial], Sequence[float]]],
+    constraints_func: Callable[[FrozenTrial], Sequence[float]] | None,
 ) -> None:
     study = optuna.create_study(directions=["minimize", "minimize", "minimize"])
     with pytest.raises(ValueError):
@@ -308,7 +309,6 @@ def test_get_pareto_front_plot(
     include_dominated_trials: bool,
     has_constraints_func: bool,
 ) -> None:
-
     info = info_template
     if not include_dominated_trials:
         info = info._replace(include_dominated_trials=False, non_best_trials_with_values=[])
@@ -320,114 +320,7 @@ def test_get_pareto_front_plot(
         figure.write_image(BytesIO())
     else:
         plt.savefig(BytesIO())
-
-
-def test_make_hovertext() -> None:
-    trial_no_user_attrs = FrozenTrial(
-        number=0,
-        trial_id=0,
-        state=TrialState.COMPLETE,
-        value=0.2,
-        datetime_start=datetime.datetime.now(),
-        datetime_complete=datetime.datetime.now(),
-        params={"x": 10},
-        distributions={"x": FloatDistribution(5, 12)},
-        user_attrs={},
-        system_attrs={},
-        intermediate_values={},
-    )
-    assert (
-        _make_hovertext(trial_no_user_attrs)
-        == dedent(
-            """
-        {
-          "number": 0,
-          "values": [
-            0.2
-          ],
-          "params": {
-            "x": 10
-          }
-        }
-        """
-        )
-        .strip()
-        .replace("\n", "<br>")
-    )
-
-    trial_user_attrs_valid_json = FrozenTrial(
-        number=0,
-        trial_id=0,
-        state=TrialState.COMPLETE,
-        value=0.2,
-        datetime_start=datetime.datetime.now(),
-        datetime_complete=datetime.datetime.now(),
-        params={"x": 10},
-        distributions={"x": FloatDistribution(5, 12)},
-        user_attrs={"a": 42, "b": 3.14},
-        system_attrs={},
-        intermediate_values={},
-    )
-    assert (
-        _make_hovertext(trial_user_attrs_valid_json)
-        == dedent(
-            """
-        {
-          "number": 0,
-          "values": [
-            0.2
-          ],
-          "params": {
-            "x": 10
-          },
-          "user_attrs": {
-            "a": 42,
-            "b": 3.14
-          }
-        }
-        """
-        )
-        .strip()
-        .replace("\n", "<br>")
-    )
-
-    trial_user_attrs_invalid_json = FrozenTrial(
-        number=0,
-        trial_id=0,
-        state=TrialState.COMPLETE,
-        value=0.2,
-        datetime_start=datetime.datetime.now(),
-        datetime_complete=datetime.datetime.now(),
-        params={"x": 10},
-        distributions={"x": FloatDistribution(5, 12)},
-        user_attrs={"a": 42, "b": 3.14, "c": np.zeros(1), "d": np.nan},
-        system_attrs={},
-        intermediate_values={},
-    )
-    assert (
-        _make_hovertext(trial_user_attrs_invalid_json)
-        == dedent(
-            """
-        {
-          "number": 0,
-          "values": [
-            0.2
-          ],
-          "params": {
-            "x": 10
-          },
-          "user_attrs": {
-            "a": 42,
-            "b": 3.14,
-            "c": "[0.]",
-            "d": NaN
-          }
-        }
-        """
-        )
-        .strip()
-        .replace("\n", "<br>")
-    )
+        plt.close()
 
 
 @pytest.mark.parametrize("direction", ["minimize", "maximize"])
