@@ -1,6 +1,9 @@
+import datetime
 import logging
+from textwrap import dedent
 from typing import cast
 
+import numpy as np
 import pytest
 from pytest import LogCaptureFixture
 
@@ -15,10 +18,10 @@ from optuna.visualization import is_available
 from optuna.visualization._utils import _check_plot_args
 from optuna.visualization._utils import _filter_nonfinite
 from optuna.visualization._utils import _is_log_scale
+from optuna.visualization._utils import _make_hovertext
 
 
 def test_is_log_scale() -> None:
-
     study = create_study()
     study.add_trial(
         create_trial(
@@ -42,7 +45,6 @@ def test_is_log_scale() -> None:
 
 
 def _is_plotly_available() -> bool:
-
     try:
         import plotly  # NOQA
 
@@ -53,12 +55,10 @@ def _is_plotly_available() -> bool:
 
 
 def test_visualization_is_available() -> None:
-
     assert is_available() == _is_plotly_available()
 
 
 def test_check_plot_args() -> None:
-
     study = create_study(directions=["minimize", "minimize"])
     with pytest.raises(ValueError):
         _check_plot_args(study, None, "Objective Value")
@@ -71,7 +71,6 @@ def test_check_plot_args() -> None:
     "value, expected", [(float("inf"), 1), (-float("inf"), 1), (float("nan"), 1), (0.0, 2)]
 )
 def test_filter_inf_trials(value: float, expected: int) -> None:
-
     study = create_study()
     study.add_trial(
         create_trial(
@@ -109,7 +108,6 @@ def test_filter_inf_trials(value: float, expected: int) -> None:
 def test_filter_inf_trials_multiobjective(
     value: float, objective_selected: int, expected: int
 ) -> None:
-
     study = create_study(directions=["minimize", "maximize"])
     study.add_trial(
         create_trial(
@@ -143,7 +141,6 @@ def test_filter_inf_trials_multiobjective(
 
 @pytest.mark.parametrize("with_message", [True, False])
 def test_filter_inf_trials_message(caplog: LogCaptureFixture, with_message: bool) -> None:
-
     study = create_study()
     study.add_trial(
         create_trial(
@@ -182,3 +179,111 @@ def test_filter_nonfinite_with_invalid_target() -> None:
     trials = study.get_trials(states=(TrialState.COMPLETE,))
     with pytest.raises(ValueError):
         _filter_nonfinite(trials, target=lambda t: "invalid target")  # type: ignore
+
+
+def test_make_hovertext() -> None:
+    trial_no_user_attrs = FrozenTrial(
+        number=0,
+        trial_id=0,
+        state=TrialState.COMPLETE,
+        value=0.2,
+        datetime_start=datetime.datetime.now(),
+        datetime_complete=datetime.datetime.now(),
+        params={"x": 10},
+        distributions={"x": FloatDistribution(5, 12)},
+        user_attrs={},
+        system_attrs={},
+        intermediate_values={},
+    )
+    assert (
+        _make_hovertext(trial_no_user_attrs)
+        == dedent(
+            """
+        {
+          "number": 0,
+          "values": [
+            0.2
+          ],
+          "params": {
+            "x": 10
+          }
+        }
+        """
+        )
+        .strip()
+        .replace("\n", "<br>")
+    )
+
+    trial_user_attrs_valid_json = FrozenTrial(
+        number=0,
+        trial_id=0,
+        state=TrialState.COMPLETE,
+        value=0.2,
+        datetime_start=datetime.datetime.now(),
+        datetime_complete=datetime.datetime.now(),
+        params={"x": 10},
+        distributions={"x": FloatDistribution(5, 12)},
+        user_attrs={"a": 42, "b": 3.14},
+        system_attrs={},
+        intermediate_values={},
+    )
+    assert (
+        _make_hovertext(trial_user_attrs_valid_json)
+        == dedent(
+            """
+        {
+          "number": 0,
+          "values": [
+            0.2
+          ],
+          "params": {
+            "x": 10
+          },
+          "user_attrs": {
+            "a": 42,
+            "b": 3.14
+          }
+        }
+        """
+        )
+        .strip()
+        .replace("\n", "<br>")
+    )
+
+    trial_user_attrs_invalid_json = FrozenTrial(
+        number=0,
+        trial_id=0,
+        state=TrialState.COMPLETE,
+        value=0.2,
+        datetime_start=datetime.datetime.now(),
+        datetime_complete=datetime.datetime.now(),
+        params={"x": 10},
+        distributions={"x": FloatDistribution(5, 12)},
+        user_attrs={"a": 42, "b": 3.14, "c": np.zeros(1), "d": np.nan},
+        system_attrs={},
+        intermediate_values={},
+    )
+    assert (
+        _make_hovertext(trial_user_attrs_invalid_json)
+        == dedent(
+            """
+        {
+          "number": 0,
+          "values": [
+            0.2
+          ],
+          "params": {
+            "x": 10
+          },
+          "user_attrs": {
+            "a": 42,
+            "b": 3.14,
+            "c": "[0.]",
+            "d": NaN
+          }
+        }
+        """
+        )
+        .strip()
+        .replace("\n", "<br>")
+    )
