@@ -1,8 +1,8 @@
 """
 .. _distributed:
 
-4. Easy Parallelization
-=======================
+Easy Parallelization
+====================
 
 It's straightforward to parallelize :func:`optuna.study.Study.optimize`.
 
